@@ -26,7 +26,6 @@
 #ifndef MODULES_ADMINAPI_COMMON_BASE_CLUSTER_IMPL_H_
 #define MODULES_ADMINAPI_COMMON_BASE_CLUSTER_IMPL_H_
 
-#include <list>
 #include <memory>
 #include <string>
 #include <string_view>
@@ -39,6 +38,7 @@
 #include "modules/adminapi/common/cluster_types.h"
 #include "modules/adminapi/common/common.h"
 #include "modules/adminapi/common/group_replication_options.h"
+#include "modules/adminapi/common/gtid_validations.h"
 #include "modules/adminapi/common/metadata_storage.h"
 #include "mysqlshdk/libs/db/connection_options.h"
 #include "mysqlshdk/libs/mysql/lock_service.h"
@@ -65,6 +65,23 @@ inline constexpr std::string_view k_cluster_attribute_member_auth_type{
 inline constexpr std::string_view k_cluster_attribute_cert_issuer{
     "opt_certIssuer"};
 
+// replication channel options
+inline constexpr std::string_view k_cluster_attribute_repl_connect_retry{
+    "opt_replConnectRetry"};
+inline constexpr std::string_view k_cluster_attribute_repl_retry_count{
+    "opt_replRetryCount"};
+inline constexpr std::string_view k_cluster_attribute_repl_heartbeat_period{
+    "opt_replHeartbeatPeriod"};
+inline constexpr std::string_view
+    k_cluster_attribute_repl_compression_algorithms{
+        "opt_replCompressionAlgorithms"};
+inline constexpr std::string_view
+    k_cluster_attribute_repl_zstd_compression_level{
+        "opt_replZstdCompressionLevel"};
+inline constexpr std::string_view k_cluster_attribute_repl_bind{"opt_replBind"};
+inline constexpr std::string_view k_cluster_attribute_repl_network_namespace{
+    "opt_replNetworkNamespace"};
+
 // Cluster capabilities
 inline constexpr const char k_cluster_capabilities[] = "capabilities";
 
@@ -95,7 +112,7 @@ class Base_cluster_impl {
     m_description = description;
   }
 
-  std::string get_name() const { return m_cluster_name; }
+  const std::string &get_name() const { return m_cluster_name; }
 
   void set_cluster_name(const std::string &name) { m_cluster_name = name; }
   const std::string &cluster_name() const { return m_cluster_name; }
@@ -110,6 +127,8 @@ class Base_cluster_impl {
     return m_primary_master;
   }
 
+  virtual bool check_valid() const;
+
   void check_preconditions(const std::string &function_name,
                            Function_availability *custom_func_avail = nullptr);
 
@@ -134,6 +153,17 @@ class Base_cluster_impl {
 
   virtual void release_primary() = 0;
 
+  // Locking methods (only cluster and clusterset supports them yet)
+  virtual mysqlshdk::mysql::Lock_scoped get_lock_shared(
+      [[maybe_unused]] std::chrono::seconds timeout = {}) {
+    return {};
+  }
+
+  virtual mysqlshdk::mysql::Lock_scoped get_lock_exclusive(
+      [[maybe_unused]] std::chrono::seconds timeout = {}) {
+    return {};
+  }
+
   virtual void disconnect();
 
   virtual shcore::Value list_routers(bool only_upgrade_required);
@@ -146,14 +176,16 @@ class Base_cluster_impl {
                                     const std::string &host,
                                     const Setup_account_options &options);
 
-  void remove_router_metadata(const std::string &router,
-                              bool lock_metadata = false);
+  virtual void remove_router_metadata(const std::string &router);
 
   void set_instance_tag(const std::string &instance_def,
                         const std::string &option, const shcore::Value &value);
 
   void set_cluster_tag(const std::string &option, const shcore::Value &value);
 
+  void set_router_tag(const std::string &router, const std::string &option,
+                      const shcore::Value &value);
+
   void set_instance_option(const std::string &instance_def,
                            const std::string &option,
                            const shcore::Value &value);
@@ -170,6 +202,14 @@ class Base_cluster_impl {
     return query_cluster_instance_auth_cert_subject(instance.get_uuid());
   }
 
+  virtual void set_routing_option(const std::string &router,
+                                  const std::string &option,
+                                  const shcore::Value &value);
+  virtual shcore::Dictionary_t routing_options(const std::string &router);
+
+  virtual shcore::Dictionary_t router_options(
+      const shcore::Option_pack_ref<Router_options_options> &options);
+
   /**
    * Get the tags for a specific Cluster/ReplicaSet
    *
@@ -202,6 +242,10 @@ class Base_cluster_impl {
                          const std::string &channel_name, int timeout,
                          bool only_received = false) const;
 
+  void sync_transactions(const mysqlshdk::mysql::IInstance &target_instance,
+                         Instance_type instance_type, int timeout,
+                         bool only_received = false) const;
+
   /**
    * Connect to the given instance specification given, while validating its
    * syntax.
@@ -223,9 +267,31 @@ class Base_cluster_impl {
       const mysqlshdk::db::Connection_options &instance_def,
       bool print_error = true, bool allow_account_override = false);
 
-  static bool verify_compatible_clone_versions(
-      const mysqlshdk::utils::Version &donor,
-      const mysqlshdk::utils::Version &recipient);
+  virtual std::vector<Instance_metadata> get_instances_from_metadata()
+      const = 0;
+
+  void create_clone_recovery_user_nobinlog(
+      mysqlshdk::mysql::IInstance *target_instance,
+      const mysqlshdk::mysql::Auth_options &donor_account,
+      const std::string &account_host, const std::string &account_cert_issuer,
+      const std::string &account_cert_subject, bool dry_run);
+
+  void handle_clone_provisioning(
+      const std::shared_ptr<mysqlsh::dba::Instance> &recipient,
+      const std::shared_ptr<mysqlsh::dba::Instance> &donor,
+      const Async_replication_options &ar_options,
+      const std::string &repl_account_host, const std::string &cert_issuer,
+      const std::string &cert_subject,
+      const Recovery_progress_style &progress_style, int sync_timeout,
+      bool dry_run);
+
+  static void check_compatible_clone_donor(
+      const mysqlshdk::mysql::IInstance &donor,
+      const mysqlshdk::mysql::IInstance &recipient);
+
+  virtual Clone_availability check_clone_availablity(
+      const mysqlshdk::mysql::IInstance &donor,
+      const mysqlshdk::mysql::IInstance &recipient) const;
 
  protected:
   Cluster_id m_id;
@@ -273,13 +339,15 @@ class Base_cluster_impl {
    * @param value the value that was provided for the option. We use it to
    * validate if built-in tags are of the expected type or can be converted to
    * it.
+   * @param built_in_tags allowed tags+types that start with _
    * @return a tuple with strings and value: namespace, option_name, value.
    * @throws argumentError if the format of the option is invalid, or using an
    * unsupported namespace.
    */
   std::tuple<std::string, std::string, shcore::Value>
   validate_set_option_namespace(const std::string &option,
-                                const shcore::Value &value) const;
+                                const shcore::Value &value,
+                                const built_in_tags_map_t &built_in_tags) const;
 
  private:
   void disconnect_internal();
