@@ -44,16 +44,16 @@ enum class Member_recovery_method { AUTO, INCREMENTAL, CLONE };
 
 struct Clone_options {
   enum Unpack_target {
-    NONE,                   // none
-    CREATE_CLUSTER,         // only disableClone
-    JOIN_CLUSTER,           // all but disableClone
-    JOIN_REPLICASET,        // ReplicaSet only options
-    CREATE_REPLICA_CLUSTER  // same as JOIN_CLUSTER
+    NONE,                    // none
+    CREATE_CLUSTER,          // only disableClone
+    JOIN_CLUSTER,            // all but disableClone
+    JOIN_REPLICASET,         // ReplicaSet only options
+    CREATE_REPLICA_CLUSTER,  // same as JOIN_CLUSTER
+    JOIN_READ_REPLICA        // Only recoveryMethod
   };
 
-  Clone_options() : target(NONE) {}
-
-  explicit Clone_options(Unpack_target t) : target(t) {}
+  explicit Clone_options() noexcept {}
+  explicit Clone_options(Unpack_target t) noexcept : target(t) {}
 
   template <typename Unpacker>
   Unpacker &unpack(Unpacker *options) {
@@ -62,15 +62,15 @@ struct Clone_options {
   }
 
   void check_option_values(const mysqlshdk::utils::Version &version,
+                           bool clone_donor_allowed = true,
                            const Cluster_impl *cluster = nullptr);
 
   void set_clone_donor(const std::string &value);
 
-  Unpack_target target;
-
+  Unpack_target target{NONE};
   std::optional<bool> disable_clone;
-  bool gtid_set_is_complete = false;
-  mysqlshdk::utils::nullable<Member_recovery_method> recovery_method;
+  bool gtid_set_is_complete{false};
+  std::optional<Member_recovery_method> recovery_method;
   std::string recovery_method_str_invalid;
   std::optional<std::string> clone_donor;
 };
@@ -125,6 +125,17 @@ struct Create_cluster_clone_options : public Clone_options {
   }
 };
 
+struct Join_read_replica_clone_options : public Join_cluster_clone_options {
+  Join_read_replica_clone_options()
+      : Join_cluster_clone_options(Unpack_target::JOIN_READ_REPLICA) {}
+  static const shcore::Option_pack_def<Join_read_replica_clone_options>
+      &options();
+
+  void set_clone_donor(const std::string &value) {
+    Clone_options::set_clone_donor(value);
+  }
+};
+
 }  // namespace dba
 }  // namespace mysqlsh
 
