@@ -1,16 +1,18 @@
 /*
- * Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2019, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms, as
- * designated in a particular file or component or in included license
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms,
+ * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ *
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -22,16 +24,15 @@
  */
 
 #include "modules/adminapi/common/global_topology.h"
-#include <algorithm>
+
 #include <list>
 #include <vector>
-#include "modules/adminapi/common/common.h"
+
 #include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/metadata_storage.h"
 #include "modules/adminapi/common/parallel_applier_options.h"
 #include "mysqlshdk/include/scripting/types.h"
 #include "mysqlshdk/include/shellcore/console.h"
-#include "mysqlshdk/libs/mysql/repl_config.h"
 #include "mysqlshdk/libs/utils/utils_net.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
@@ -117,10 +118,9 @@ void load_instance_channels(Instance *instance,
       if (!mysqlshdk::mysql::get_channel_info(
               *conn, ch.channel_name, &instance->master_channel->master_info,
               &instance->master_channel->relay_log_info)) {
-        throw shcore::Exception::runtime_error(
-            "Could not query replication configuration from "
-            "mysql.slave_master_info, relay_log_info (channel = '" +
-            ch.channel_name + "')");
+        throw shcore::Exception::runtime_error(shcore::str_format(
+            "Could not query replication configuration for channel '%s'.",
+            ch.channel_name.c_str()));
       }
     } else if (ch.channel_name != "group_replication_applier") {
       log_warning("%s has an unmanaged/unexpected replication channel '%s'",
@@ -282,7 +282,7 @@ void Server::scan(const mysqlshdk::mysql::IInstance *conn,
     // record all slaves as unmanaged for a start, since we don't know their
     // corresponding channel name before we scan them. The unmanaged list is
     // updated after determining that a slave is a managed replica.
-    auto slaves = mysqlshdk::mysql::get_slaves(*conn);
+    auto slaves = mysqlshdk::mysql::get_replicas(*conn);
     std::copy(slaves.begin(), slaves.end(),
               std::back_inserter(instance.unmanaged_replicas));
     if (!instance.unmanaged_replicas.empty()) {
@@ -345,24 +345,23 @@ void Global_topology::load_instance_state(Instance *instance,
 
 void Global_topology::load_instance_slaves(Instance *instance,
                                            mysqlsh::dba::Instance *conn) {
-  std::vector<mysqlshdk::mysql::Slave_host> slaves(
-      mysqlshdk::mysql::get_slaves(*conn));
-  if (!slaves.empty()) {
-    log_info("%s has %zi instances replicating from it",
-             instance->label.c_str(), slaves.size());
+  auto slaves = mysqlshdk::mysql::get_replicas(*conn);
+  if (slaves.empty()) return;
 
-    for (const auto &slave : slaves) {
-      try {
-        find_member(slave.uuid);
-        // if member is already known, ignore it
-      } catch (...) {
-        // if member is not known, log and include it in the instance's info
-        log_info("%s has an unmanaged replica instance %s:%i (%s)",
-                 instance->label.c_str(), slave.host.c_str(), slave.port,
-                 slave.uuid.c_str());
+  log_info("%s has %zi instances replicating from it", instance->label.c_str(),
+           slaves.size());
 
-        instance->unmanaged_replicas.push_back(slave);
-      }
+  for (const auto &slave : slaves) {
+    try {
+      find_member(slave.uuid);
+      // if member is already known, ignore it
+    } catch (...) {
+      // if member is not known, log and include it in the instance's info
+      log_info("%s has an unmanaged replica instance %s:%i (%s)",
+               instance->label.c_str(), slave.host.c_str(), slave.port,
+               slave.uuid.c_str());
+
+      instance->unmanaged_replicas.push_back(slave);
     }
   }
 }
@@ -559,8 +558,7 @@ void Server_global_topology::load_cluster(MetadataStorage *metadata,
                                           const Cluster_metadata &cluster) {
   m_topology_type = cluster.async_topology_type;
 
-  std::vector<Instance_metadata> instances(
-      metadata->get_replica_set_instances(cluster.cluster_id));
+  auto instances = metadata->get_replica_set_instances(cluster.cluster_id);
   log_debug("%zi instances in replicaset %s", instances.size(),
             cluster.full_cluster_name().c_str());
 
@@ -751,13 +749,32 @@ Server *Server_global_topology::scan_instance_recursive(
   }
 
   // Run discovery on slaves
-  std::list<mysqlshdk::mysql::Slave_host>::iterator next_it =
-      member->unmanaged_replicas.begin();
-  while (next_it != member->unmanaged_replicas.end()) {
+  for (auto next_it = member->unmanaged_replicas.begin();
+       next_it != member->unmanaged_replicas.end();) {
     auto slave_it = next_it++;
     const auto &slave = *slave_it;
 
-    std::string endpoint =
+    if (!slave.has_valid_endpoint()) {
+      console->print_warning(shcore::str_format(
+          "Instance '%s' (a replica of '%s') doesn't have the 'report_host' "
+          "variable properly configured: '%s'",
+          slave.uuid.c_str(), instance->descr().c_str(),
+          mysqlshdk::utils::make_host_and_port(slave.host, slave.port)
+              .c_str()));
+
+      console->print_info(shcore::str_format(
+          "In order to discover all available replicas, each one must have the "
+          "'report_host' variable set, otherwise they can't be uniquely "
+          "identified and reached from '%s'. Please configure the "
+          "'report_host' variable of instance '%s'.",
+          instance->descr().c_str(), slave.uuid.c_str()));
+
+      member->invalid_replicas.push_back(slave);
+      member->unmanaged_replicas.erase(slave_it);
+      continue;
+    }
+
+    auto endpoint =
         mysqlshdk::utils::make_host_and_port(slave.host, slave.port);
 
     // Connect to the slave and scan it
@@ -828,9 +845,10 @@ void Server_global_topology::show_raw() const {
       console->print_info("    - Unreachable: " + instance->connect_error);
       continue;
     }
+
     if (instance->master_channel) {
       console->print_info(shcore::str_format(
-          "    - replicates from %s",
+          "    - replicates from '%s'",
           instance->master_channel->master_ptr->label.c_str()));
       console->print_info(
           "\t" + shcore::str_join(shcore::str_split(
@@ -842,36 +860,37 @@ void Server_global_topology::show_raw() const {
 
     for (const auto &channel : instance->unmanaged_channels) {
       console->print_info(shcore::str_format(
-          "    - replicates from %s:%i through unsupported channel '%s'",
-          channel.host.c_str(), channel.port, channel.channel_name.c_str()));
+          "    - replicates from '%s' through unsupported channel '%s'",
+          mysqlshdk::utils::make_host_and_port(channel.host, channel.port)
+              .c_str(),
+          channel.channel_name.c_str()));
     }
+
     for (const auto &slave : instance->unmanaged_replicas) {
-      console->print_info(
-          shcore::str_format("    - has a replica %s:%i through an unsupported "
-                             "replication channel",
-                             slave.host.c_str(), slave.port));
+      console->print_info(shcore::str_format(
+          "    - has a replica '%s' through an unsupported replication "
+          "channel",
+          mysqlshdk::utils::make_host_and_port(slave.host, slave.port)
+              .c_str()));
     }
   }
 }
 
 // -----------------------------------------------------------------------------
 
-Global_topology *scan_global_topology(MetadataStorage *metadata,
-                                      const Cluster_metadata &cmd,
-                                      const std::string &channel_name,
-                                      bool deep) {
-  if (cmd.type == Cluster_type::GROUP_REPLICATION) {
+std::unique_ptr<Global_topology> scan_global_topology(
+    MetadataStorage *metadata, const Cluster_metadata &cmd,
+    const std::string &channel_name, bool deep) {
+  if (cmd.type == Cluster_type::GROUP_REPLICATION)
     throw std::logic_error("internal error");
-  } else {
-    std::unique_ptr<Server_global_topology> topo(
-        new Server_global_topology(channel_name));
 
-    topo->load_cluster(metadata, cmd);
-    topo->check_servers(deep);
-    if (deep) topo->check_gtid_consistency();
+  auto topo = std::make_unique<Server_global_topology>(channel_name);
 
-    return topo.release();
-  }
+  topo->load_cluster(metadata, cmd);
+  topo->check_servers(deep);
+  if (deep) topo->check_gtid_consistency();
+
+  return topo;
 }
 
 }  // namespace topology
