@@ -27,12 +27,16 @@
 
 #include <string>
 
+#include <mysqld_error.h>
+
 #include "modules/adminapi/common/accounts.h"
 #include "modules/adminapi/common/common.h"
 #include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/instance_validations.h"
+#include "modules/adminapi/common/topology_executor.h"
 #include "modules/adminapi/dba/check_instance.h"
 #include "mysqlshdk/include/shellcore/console.h"
+#include "mysqlshdk/libs/mysql/group_replication.h"
 #include "mysqlshdk/libs/mysql/replication.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 
@@ -51,11 +55,11 @@ void ensure_gr_instance_configuration_valid(
         "Validating instance configuration at " +
         target_instance->get_connection_options().uri_endpoint() + "...");
 
-    Check_instance check(target_instance->get_connection_options(), "", true,
-                         skip_check_tables_pk);
-    check.prepare();
-    result = check.execute();
-    check.finish();
+    result =
+        Topology_executor<Check_instance>{
+            target_instance->get_connection_options(), "", true,
+            skip_check_tables_pk}
+            .run();
 
     if (!result || result.as_map()->at("status").get_string() == "ok") {
       console->print_info("Instance configuration is suitable.");
@@ -90,7 +94,7 @@ void validate_ar_configuration(mysqlshdk::mysql::IInstance *target_instance) {
       target_instance, mysqlshdk::config::k_dft_cfg_server_handler, true);
 
   if (!checks::validate_configuration(
-           target_instance, "", cfg.get(), Cluster_type::ASYNC_REPLICATION,
+           *target_instance, "", cfg.get(), Cluster_type::ASYNC_REPLICATION,
            true, &restart, &config_file_change, &sysvar_change, &ret_val)
            .empty()) {
     if (restart && !config_file_change && !sysvar_change) {
