@@ -1,16 +1,18 @@
 /*
- * Copyright (c) 2015, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms, as
- * designated in a particular file or component or in included license
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms,
+ * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ *
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -20,21 +22,22 @@
  * along with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
+
 #include "modules/devapi/mod_mysqlx_table_select.h"
+
 #include <memory>
 #include <string>
 #include <vector>
+
 #include "modules/devapi/base_constants.h"
 #include "modules/devapi/mod_mysqlx_resultset.h"
 #include "modules/devapi/mod_mysqlx_table.h"
-#include "scripting/common.h"
 #include "shellcore/utils_help.h"
 
 namespace mysqlsh {
 namespace mysqlx {
 
 using shcore::Value;
-using std::placeholders::_1;
 
 REGISTER_HELP_CLASS(TableSelect, mysqlx);
 REGISTER_HELP(TABLESELECT_BRIEF, "Operation to retrieve rows from a table.");
@@ -44,6 +47,7 @@ REGISTER_HELP(TABLESELECT_DETAIL,
               "<b>Table</b> class.");
 TableSelect::TableSelect(std::shared_ptr<Table> owner)
     : Table_crud_definition(std::static_pointer_cast<DatabaseObject>(owner)) {
+  using namespace std::placeholders;
   message_.mutable_collection()->set_schema(owner->schema()->name());
   message_.mutable_collection()->set_name(owner->name());
   message_.set_data_model(Mysqlx::Crud::TABLE);
@@ -790,12 +794,12 @@ TableSelect TableSelect::offset(int numberOfRows) {}
 void TableSelect::set_lock_contention(const shcore::Argument_list &args) {
   std::string lock_contention;
   if (args.size() == 1) {
-    if (args[0].type == shcore::Object) {
+    if (args[0].get_type() == shcore::Object) {
       std::shared_ptr<Constant> constant =
           std::dynamic_pointer_cast<Constant>(args.object_at(0));
       if (constant && constant->group() == "LockContention")
         lock_contention = constant->data().get_string();
-    } else if (args[0].type == shcore::String) {
+    } else if (args[0].get_type() == shcore::String) {
       lock_contention = args.string_at(0);
     }
 
@@ -1245,7 +1249,7 @@ shcore::Value TableSelect::execute(const shcore::Argument_list &args) {
     }));
 
     update_functions(F::execute);
-    if (!m_limit.is_null()) enable_function(F::offset);
+    if (m_limit.has_value()) enable_function(F::offset);
   }
   CATCH_AND_TRANSLATE_CRUD_EXCEPTION(get_function_name("execute"));
 
