@@ -220,7 +220,7 @@ shcore::Value TableUpdate::set(const shcore::Argument_list &args) {
 
     // Only expression objects are allowed as values
     std::string expr_data;
-    if (args[1].type == shcore::Object) {
+    if (args[1].get_type() == shcore::Object) {
       shcore::Object_bridge_ref object = args.object_at(1);
 
       auto expression = std::dynamic_pointer_cast<Expression>(object);
