@@ -162,9 +162,8 @@ class SHCORE_PUBLIC Shell : public shcore::Cpp_object_bridge
 #endif
 
   shcore::Array_t list_credential_helpers();
-  void store_credential(
-      const std::string &url,
-      const mysqlshdk::utils::nullable<std::string> &password);
+  void store_credential(const std::string &url,
+                        std::optional<std::string> password);
   void delete_credential(const std::string &url);
   void delete_all_credentials();
   shcore::Array_t list_credentials();
