@@ -35,16 +35,17 @@
 #include <algorithm>
 #include <iterator>
 
+#include "modules/util/common/dump/utils.h"
 #include "mysqlshdk/include/scripting/type_info/custom.h"
 #include "mysqlshdk/include/scripting/type_info/generic.h"
 #include "mysqlshdk/include/shellcore/console.h"
+#include "mysqlshdk/libs/db/mysql/result.h"
 #include "mysqlshdk/libs/utils/strformat.h"
+#include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_lexing.h"
 #include "mysqlshdk/libs/utils/utils_sqlstring.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
-#include "modules/util/common/dump/utils.h"
-
 namespace mysqlsh {
 namespace dump {
 
@@ -66,6 +67,11 @@ const shcore::Option_pack_def<Dump_options> &Dump_options::options() {
   return opts;
 }
 
+const mysqlshdk::utils::Version &Dump_options::current_version() {
+  static const auto k_current_version = mysqlshdk::utils::Version(MYSH_VERSION);
+  return k_current_version;
+}
+
 void Dump_options::on_start_unpack(const shcore::Dictionary_t &options) {
   m_options = options;
 }
@@ -82,7 +88,8 @@ void Dump_options::set_string_option(const std::string &option,
           "The option 'compression' cannot be set to an empty string.");
     }
 
-    m_compression = mysqlshdk::storage::to_compression(value);
+    m_compression =
+        mysqlshdk::storage::to_compression(value, &m_compression_options);
   } else {
     // This function should only be called with the options above.
     assert(false);
@@ -90,8 +97,8 @@ void Dump_options::set_string_option(const std::string &option,
 }
 
 void Dump_options::set_storage_config(
-    const std::shared_ptr<mysqlshdk::storage::Config> &storage_config) {
-  m_storage_config = storage_config;
+    std::shared_ptr<mysqlshdk::storage::Config> storage_config) {
+  m_storage_config = std::move(storage_config);
 }
 
 void Dump_options::on_log_options(const char *msg) const {
@@ -190,7 +197,7 @@ void Dump_options::set_where_clause(
     schema.clear();
     table.clear();
 
-    common::parse_schema_and_object(w.first,
+    shcore::parse_schema_and_object(w.first,
                                     "table name key of the 'where' option",
                                     "table", &schema, &table);
 
@@ -244,7 +251,7 @@ void Dump_options::set_partitions(
     schema.clear();
     table.clear();
 
-    common::parse_schema_and_object(p.first,
+    shcore::parse_schema_and_object(p.first,
                                     "table name key of the 'partitions' option",
                                     "table", &schema, &table);
 
@@ -324,5 +331,29 @@ void Dump_options::validate_partitions() const {
   }
 }
 
+void Dump_options::set_target_version(const mysqlshdk::utils::Version &version,
+                                      bool validate) {
+  if (validate) {
+    const auto k_minimum_version = mysqlshdk::utils::Version(8, 0, 25);
+
+    if (version > current_version()) {
+      throw std::invalid_argument("Requested MySQL version '" +
+                                  version.get_base() +
+                                  "' is newer than the maximum version '" +
+                                  current_version().get_base() +
+                                  "' supported by this version of MySQL Shell");
+    } else if (version < k_minimum_version) {
+      // 8.0.25 is the minimum MDS version we support
+      throw std::invalid_argument("Requested MySQL version '" +
+                                  version.get_base() +
+                                  "' is older than the minimum version '" +
+                                  k_minimum_version.get_base() +
+                                  "' supported by this version of MySQL Shell");
+    }
+  }
+
+  m_target_version = version;
+}
+
 }  // namespace dump
 }  // namespace mysqlsh
