@@ -37,13 +37,13 @@
 
 #include "mysqlshdk/include/scripting/types.h"
 #include "mysqlshdk/include/scripting/types_cpp.h"
+#include "mysqlshdk/libs/db/filtering_options.h"
 #include "mysqlshdk/libs/db/session.h"
 #include "mysqlshdk/libs/storage/compressed_file.h"
 #include "mysqlshdk/libs/storage/config.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/version.h"
 
-#include "modules/util/common/dump/filtering_options.h"
 #include "modules/util/dump/compatibility_option.h"
 #include "modules/util/dump/instance_cache.h"
 #include "modules/util/import_table/dialect.h"
@@ -51,8 +51,15 @@
 namespace mysqlsh {
 namespace dump {
 
+enum class Dry_run {
+  DISABLED,
+  WRITE_EMPTY_DATA_FILES,
+  DONT_WRITE_ANY_FILES,
+};
+
 class Dump_options {
  public:
+  using Filtering_options = mysqlshdk::db::Filtering_options;
   Dump_options();
 
   Dump_options(const Dump_options &) = default;
@@ -65,16 +72,31 @@ class Dump_options {
 
   static const shcore::Option_pack_def<Dump_options> &options();
 
+  static const mysqlshdk::utils::Version &current_version();
+
   void validate() const;
 
   // setters
+  virtual void set_output_url(const std::string &url) { m_output_url = url; }
+
   void set_session(const std::shared_ptr<mysqlshdk::db::ISession> &session);
 
+  void set_compression(mysqlshdk::storage::Compression compression) {
+    m_compression = compression;
+  }
+
+  void set_storage_config(
+      std::shared_ptr<mysqlshdk::storage::Config> storage_config);
+
+  void set_dry_run_mode(Dry_run dry_run) { m_dry_run_mode = dry_run; }
+
+  void disable_index_files() { m_write_index_files = false; }
+
+  void dont_rename_data_files() { m_rename_data_files = false; }
+
   // getters
   const std::string &output_url() const { return m_output_url; }
 
-  void set_output_url(const std::string &url) { m_output_url = url; }
-
   const shcore::Dictionary_t &original_options() const { return m_options; }
 
   bool use_base64() const { return m_use_base64; }
@@ -85,6 +107,10 @@ class Dump_options {
 
   mysqlshdk::storage::Compression compression() const { return m_compression; }
 
+  const mysqlshdk::storage::Compression_options &compression_options() const {
+    return m_compression_options;
+  }
+
   const std::shared_ptr<mysqlshdk::db::ISession> &session() const {
     return m_session;
   }
@@ -97,19 +123,25 @@ class Dump_options {
 
   const std::string &character_set() const { return m_character_set; }
 
-  const std::optional<mysqlshdk::utils::Version> &mds_compatibility() const {
-    return m_mds;
-  }
+  bool mds_compatibility() const { return m_is_mds; }
 
   const Compatibility_options &compatibility_options() const {
     return m_compatibility_options;
   }
 
-  common::Filtering_options &filters() { return m_filtering_options; }
-  const common::Filtering_options &filters() const {
-    return m_filtering_options;
+  Filtering_options &filters() { return m_filtering_options; }
+  const Filtering_options &filters() const { return m_filtering_options; }
+
+  const mysqlshdk::utils::Version &target_version() const {
+    if (m_target_version.has_value()) {
+      return *m_target_version;
+    } else {
+      return current_version();
+    }
   }
 
+  bool implicit_target_version() const { return !m_target_version.has_value(); }
+
   const Instance_cache_builder::Partition_filters &included_partitions() const {
     return m_partitions;
   }
@@ -117,12 +149,22 @@ class Dump_options {
   const std::string &where(const std::string &schema,
                            const std::string &table) const;
 
+  Dry_run dry_run_mode() const { return m_dry_run_mode; }
+
+  bool is_dry_run() const { return Dry_run::DISABLED != m_dry_run_mode; }
+
+  bool write_index_files() const { return m_write_index_files; }
+
+  bool rename_data_files() const { return m_rename_data_files; }
+
   virtual bool split() const = 0;
 
   virtual uint64_t bytes_per_chunk() const = 0;
 
   virtual std::size_t threads() const = 0;
 
+  virtual std::size_t worker_threads() const { return threads(); }
+
   virtual bool is_export_only() const = 0;
 
   virtual bool use_single_file() const = 0;
@@ -131,12 +173,12 @@ class Dump_options {
 
   virtual bool dump_data() const = 0;
 
-  virtual bool is_dry_run() const = 0;
-
   virtual bool consistent_dump() const = 0;
 
   virtual bool skip_consistency_checks() const = 0;
 
+  virtual bool skip_upgrade_checks() const = 0;
+
   virtual bool dump_events() const = 0;
 
   virtual bool dump_routines() const = 0;
@@ -149,22 +191,18 @@ class Dump_options {
 
   virtual bool dump_binlog_info() const = 0;
 
-  virtual bool par_manifest() const = 0;
+  virtual bool checksum() const = 0;
 
  protected:
-  void set_compression(mysqlshdk::storage::Compression compression) {
-    m_compression = compression;
-  }
-
-  void set_mds_compatibility(
-      const std::optional<mysqlshdk::utils::Version> &mds) {
-    m_mds = mds;
-  }
+  void enable_mds_compatibility() { m_is_mds = true; }
 
   void set_compatibility_option(Compatibility_option c) {
     m_compatibility_options |= c;
   }
 
+  void set_target_version(const mysqlshdk::utils::Version &version,
+                          bool validate = true);
+
   void set_where_clause(const std::map<std::string, std::string> &where);
 
   void set_where_clause(const std::string &schema, const std::string &table,
@@ -188,14 +226,12 @@ class Dump_options {
       const std::unordered_set<std::string> &tables) const;
 
   // currently used by dumpTables(), dumpSchemas() and dumpInstance()
-  common::Filtering_options m_filtering_options;
+  Filtering_options m_filtering_options;
 
   mutable bool m_filter_conflicts = false;
 
  protected:
   void on_start_unpack(const shcore::Dictionary_t &options);
-  void set_storage_config(
-      const std::shared_ptr<mysqlshdk::storage::Config> &storage_config);
 
   // This function should be implemented when the validation process requires
   // data NOT coming on the user options, i.e. a session
@@ -232,6 +268,7 @@ class Dump_options {
   bool m_show_progress;
   mysqlshdk::storage::Compression m_compression =
       mysqlshdk::storage::Compression::ZSTD;
+  mysqlshdk::storage::Compression_options m_compression_options;
   mysqlshdk::storage::Config_ptr m_storage_config;
 
   std::string m_character_set = "utf8mb4";
@@ -239,6 +276,12 @@ class Dump_options {
   import_table::Dialect m_dialect;
   import_table::Dialect m_dialect_unpacker;
 
+  Dry_run m_dry_run_mode = Dry_run::DISABLED;
+
+  bool m_write_index_files = true;
+
+  bool m_rename_data_files = true;
+
   // schema -> table -> condition
   std::unordered_map<std::string, std::unordered_map<std::string, std::string>>
       m_where;
@@ -250,8 +293,9 @@ class Dump_options {
   // a reference
 
   // currently used by dumpTables(), dumpSchemas() and dumpInstance()
-  std::optional<mysqlshdk::utils::Version> m_mds;
+  bool m_is_mds = false;
   Compatibility_options m_compatibility_options;
+  std::optional<mysqlshdk::utils::Version> m_target_version;
 };
 
 }  // namespace dump
