@@ -32,21 +32,22 @@
 #include <unistd.h>
 #endif
 
+#include <cassert>
 #include <map>
 #include <memory>
 #include <optional>
 #include <string>
+#include <variant>
 #include <vector>
 
 #include "modules/util/import_table/dialect.h"
-#include "mysqlshdk/include/scripting/type_info/custom.h"
-#include "mysqlshdk/include/scripting/type_info/generic.h"
 #include "mysqlshdk/include/scripting/types_cpp.h"
 #include "mysqlshdk/libs/aws/s3_bucket_options.h"
 #include "mysqlshdk/libs/azure/blob_storage_options.h"
 #include "mysqlshdk/libs/db/connection_options.h"
 #include "mysqlshdk/libs/db/mysql/session.h"
 #include "mysqlshdk/libs/oci/oci_bucket_options.h"
+#include "mysqlshdk/libs/storage/compressed_file.h"
 #include "mysqlshdk/libs/storage/config.h"
 #include "mysqlshdk/libs/storage/ifile.h"
 
@@ -55,6 +56,12 @@ namespace import_table {
 
 using Connection_options = mysqlshdk::db::Connection_options;
 
+enum class Duplicate_handling {
+  Default,
+  Ignore,
+  Replace,
+};
+
 class Import_table_option_pack {
  public:
   Import_table_option_pack()
@@ -72,11 +79,11 @@ class Import_table_option_pack {
   static const shcore::Option_pack_def<Import_table_option_pack> &options();
   void set_decode_columns(const shcore::Dictionary_t &decode_columns);
 
-  Dialect dialect() const { return m_dialect; }
+  const Dialect &dialect() const { return m_dialect; }
 
-  bool is_multifile() const;
+  bool is_multifile() const noexcept { return m_is_multifile; }
 
-  bool is_compressed(const std::string &path) const;
+  static bool is_compressed(const std::string &path);
 
   void set_filenames(const std::vector<std::string> &filenames);
 
@@ -84,13 +91,24 @@ class Import_table_option_pack {
     return m_filelist_from_user;
   }
 
+  const std::string &single_file() const {
+    assert(!is_multifile());
+    return m_filelist_from_user[0];
+  }
+
   uint64_t max_rate() const;
 
-  bool replace_duplicates() const { return m_replace_duplicates; }
+  Duplicate_handling duplicate_handling() const { return m_duplicate_handling; }
 
-  void set_replace_duplicates(bool flag) { m_replace_duplicates = flag; }
+  void set_duplicate_handling(Duplicate_handling flag) {
+    m_duplicate_handling = flag;
+  }
 
-  const shcore::Array_t &columns() const { return m_columns; }
+  const std::vector<std::variant<std::string, uint64_t>> &columns() const {
+    return m_columns;
+  }
+
+  void clear_columns() { m_columns.clear(); }
 
   const std::map<std::string, std::string> &decode_columns() const {
     return m_decode_columns;
@@ -100,10 +118,14 @@ class Import_table_option_pack {
 
   uint64_t skip_rows_count() const { return m_skip_rows_count; }
 
+  void clear_skip_rows_count() const { m_skip_rows_count = 0; }
+
   int64_t threads_size() const { return m_threads_size; }
 
   const std::string &table() const { return m_table; }
 
+  void set_table(const std::string &table) { m_table = table; }
+
   const std::string &schema() const { return m_schema; }
 
   const std::string &partition() const { return m_partition; }
@@ -112,8 +134,6 @@ class Import_table_option_pack {
 
   const std::string &character_set() const { return m_character_set; }
 
-  size_t file_size() const { return m_file_size; }
-
   uint64_t bytes_per_chunk() const;
 
   size_t max_transaction_size() const;
@@ -144,12 +164,13 @@ class Import_table_option_pack {
   void set_bytes_per_chunk(const std::string &value);
   void set_max_rate(const std::string &value);
   void on_unpacked_options();
+  bool check_if_multifile();
+  void set_replace_duplicates(bool flag);
+  void set_columns(const shcore::Array_t &columns);
 
  protected:
-  size_t calc_thread_size();
-
   std::vector<std::string> m_filelist_from_user;
-  size_t m_file_size;
+  bool m_is_multifile = false;
   std::string m_table;
   std::string m_schema;
   std::string m_partition;
@@ -157,12 +178,12 @@ class Import_table_option_pack {
   int64_t m_threads_size = 8;
   std::optional<uint64_t> m_bytes_per_chunk;
   size_t m_max_bytes_per_transaction = 0;
-  shcore::Array_t m_columns;
+  std::vector<std::variant<std::string, uint64_t>> m_columns;
   std::map<std::string, std::string> m_decode_columns;
-  bool m_replace_duplicates = false;
+  Duplicate_handling m_duplicate_handling = Duplicate_handling::Ignore;
   uint64_t m_max_rate = 0;
   bool m_show_progress = isatty(fileno(stdout)) ? true : false;
-  uint64_t m_skip_rows_count = 0;
+  mutable uint64_t m_skip_rows_count = 0;
   Dialect m_dialect;
   mysqlshdk::oci::Oci_bucket_options m_oci_bucket_options;
   mysqlshdk::aws::S3_bucket_options m_s3_bucket_options;
@@ -177,19 +198,23 @@ class Import_table_options : public Import_table_option_pack {
  public:
   Import_table_options() = default;
   explicit Import_table_options(const Import_table_option_pack &pack);
+
+  static Import_table_options unpack(const shcore::Dictionary_t &options);
+
   void set_base_session(
       const std::shared_ptr<mysqlshdk::db::ISession> &session) {
     m_base_session = session;
   }
 
-  mysqlshdk::Masked_string full_path() const {
-    return m_file_handle->full_path();
+  size_t file_size() const {
+    assert(!is_multifile());
+    return m_file_size;
   }
 
-  /**
-   * Returns the raw pointer to the file handle
-   */
-  mysqlshdk::storage::IFile *file_handle() const { return m_file_handle.get(); }
+  const std::string &masked_full_path() const {
+    assert(!is_multifile());
+    return m_full_path;
+  }
 
   void validate();
 
@@ -197,9 +222,25 @@ class Import_table_options : public Import_table_option_pack {
 
   std::string target_import_info() const;
 
+  bool dialect_supports_chunking() const;
+
+  mysqlshdk::storage::Compression compression() const noexcept {
+    return m_compression;
+  }
+
+  // Compression is only supported by the BULK LOAD
+  void set_compression(mysqlshdk::storage::Compression compression) noexcept {
+    m_compression = compression;
+  }
+
  private:
+  size_t calc_thread_size();
+
   std::shared_ptr<mysqlshdk::db::ISession> m_base_session;
-  std::unique_ptr<mysqlshdk::storage::IFile> m_file_handle;
+  size_t m_file_size;
+  std::string m_full_path;
+  mysqlshdk::storage::Compression m_compression =
+      mysqlshdk::storage::Compression::NONE;
 };
 
 }  // namespace import_table
