@@ -40,14 +40,21 @@ using off64_t = off_t;
 #endif
 
 #include <mysql.h>
+
 #include <algorithm>
+#include <cassert>
+#include <cinttypes>
 #include <memory>
+#include <type_traits>
 #include <utility>
+
 #include "modules/util/import_table/helpers.h"
 #include "mysqlshdk/include/shellcore/console.h"
 #include "mysqlshdk/include/shellcore/scoped_contexts.h"
 #include "mysqlshdk/include/shellcore/shell_init.h"
 #include "mysqlshdk/libs/rest/error.h"
+#include "mysqlshdk/libs/storage/backend/in_memory/synchronized_file.h"
+#include "mysqlshdk/libs/storage/backend/in_memory/virtual_file.h"
 #include "mysqlshdk/libs/utils/utils_path.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
@@ -76,7 +83,7 @@ int local_infile_error_nop(void * /* userdata */, char * /* error_msg */,
 
 }  // namespace
 
-Transaction_buffer::Transaction_buffer(Dialect dialect,
+Transaction_buffer::Transaction_buffer(const Dialect &dialect,
                                        mysqlshdk::storage::IFile *file,
                                        uint64_t max_transaction_size,
                                        uint64_t skip_bytes)
@@ -84,7 +91,11 @@ Transaction_buffer::Transaction_buffer(Dialect dialect,
   m_options.max_trx_size = max_transaction_size;
   m_options.skip_bytes = skip_bytes;
 
-  if (m_dialect == Dialect::default_()) {
+  if (m_dialect.lines_terminated_by.empty() ||
+      m_dialect.fields_terminated_by == m_dialect.lines_terminated_by) {
+    // we're unable to sub-chunk properly in this case
+    m_options.max_trx_size = 0;
+  } else if (m_dialect == Dialect::default_()) {
     find_first_row_boundary_after =
         &Transaction_buffer::find_first_row_boundary_after_impl_default;
     find_last_row_boundary_before =
@@ -198,6 +209,10 @@ int Transaction_buffer::read(char *buffer, unsigned int length) {
     return m_file->read(buffer, length);
   }
 
+  if (m_options.fast_sub_chunking) {
+    return fast_sub_chunking(buffer, length);
+  }
+
   const auto read_more = [this](unsigned int count) -> int64_t {
     int64_t bytes = 0;
 
@@ -434,6 +449,75 @@ uint64_t Transaction_buffer::find_last_row_boundary_before_impl_escape(
 
   return 0;
 }
+
+int Transaction_buffer::fast_sub_chunking(char *buffer, unsigned int length) {
+  if (!m_data.empty()) {
+    // if data is stored locally, then a row didn't fit into an empty buffer,
+    // consume it first
+    return consume(buffer, length);
+  }
+
+  if (trx_bytes_left() <= 0) {
+    // the row we had stored was longer than the transaction size, or we read
+    // exactly that many bytes
+    return 0;
+  }
+
+  auto bytes =
+      m_file->read(buffer, std::min<uint64_t>(length, trx_bytes_left()));
+
+  if (0 == bytes) {
+    m_eof = true;
+    return bytes;
+  }
+
+  if (bytes < 0) {
+    // it was not possible to fit a single row into the buffer
+    if (m_trx_size) {
+      // we have some data in transaction, finish it, we'll see if this row fits
+      // into an empty buffer
+      return 0;
+    } else {
+      // row doesn't fit into an empty buffer, store it locally
+      const auto virtual_file =
+          dynamic_cast<mysqlshdk::storage::in_memory::Virtual_file *>(m_file);
+      assert(virtual_file);
+
+      if (!virtual_file) {
+        throw std::logic_error(
+            "Fast sub-chunking is only possible with virtual FS");
+      }
+
+      const auto handle =
+          dynamic_cast<mysqlshdk::storage::in_memory::Synchronized_file *>(
+              virtual_file->file());
+      assert(handle);
+
+      if (!handle) {
+        throw std::logic_error(
+            "Fast sub-chunking is only possible with synchronized I/O");
+      }
+
+      const auto row_length = handle->pending_write_size();
+
+      m_data.resize(row_length);
+      bytes = m_file->read(m_data.data(), row_length);
+
+      // this read should succeed
+      assert(static_cast<std::size_t>(bytes) == row_length);
+
+      if (bytes <= 0) {
+        return bytes;
+      }
+
+      return consume(buffer, length);
+    }
+  }
+
+  m_trx_size += bytes;
+  return bytes;
+}
+
 // ------
 
 int local_infile_init(void **buffer, const char * /* filename */,
@@ -500,14 +584,21 @@ int local_infile_read(void *userdata, char *buffer,
     file_info->rate_limit.throttle(bytes);
   }
 
-  if (*file_info->user_interrupt) {
+  if (file_info->user_interrupt->test()) {
     return -1;
   }
 
   return bytes;
 }
 
-void local_infile_end(void *) noexcept {}
+void local_infile_end(void *userdata) noexcept {
+  assert(userdata);
+  const auto file_info = static_cast<File_info *>(userdata);
+
+  if (file_info->on_infile_read_end) {
+    file_info->on_infile_read_end();
+  }
+}
 
 int local_infile_error(void *userdata, char *error_msg,
                        unsigned int error_msg_len) noexcept {
@@ -522,7 +613,7 @@ int local_infile_error(void *userdata, char *error_msg,
     } catch (...) {
       snprintf(error_msg, error_msg_len, "Unknown exception during LOAD DATA");
     }
-  } else if (file_info->user_interrupt && *file_info->user_interrupt) {
+  } else if (file_info->user_interrupt->test()) {
     snprintf(error_msg, error_msg_len, "Interrupted");
   } else {
     snprintf(error_msg, error_msg_len, "Unknown error during LOAD DATA");
@@ -532,20 +623,29 @@ int local_infile_error(void *userdata, char *error_msg,
 
 Load_data_worker::Load_data_worker(
     const Import_table_options &options, int64_t thread_id,
-    std::atomic<size_t> *prog_sent_bytes, std::atomic<size_t> *prog_file_bytes,
-    volatile bool *interrupt,
+    std::atomic<size_t> *prog_data_bytes, std::atomic<size_t> *prog_file_bytes,
+    shcore::atomic_flag *interrupt,
     shcore::Synchronized_queue<File_import_info> *range_queue,
-    std::vector<std::exception_ptr> *thread_exception, Stats *stats,
+    std::exception_ptr *thread_exception, Stats *stats,
     const std::string &query_comment)
     : m_opt(options),
       m_thread_id(thread_id),
-      m_prog_sent_bytes(prog_sent_bytes),
+      m_prog_data_bytes(prog_data_bytes),
       m_prog_file_bytes(prog_file_bytes),
-      m_interrupt(*interrupt),
+      m_interrupt(interrupt),
       m_range_queue(range_queue),
-      m_thread_exception(*thread_exception),
-      m_stats(*stats),
-      m_query_comment(query_comment) {}
+      m_thread_exception(thread_exception),
+      m_stats(stats),
+      m_query_comment(query_comment) {
+  assert(m_prog_data_bytes);
+  assert(m_prog_file_bytes);
+  assert(m_interrupt);
+  assert(m_thread_exception);
+  assert(m_stats);
+
+  m_state = Thread_state::IDLE;
+  ++m_stats->thread_states[m_state];
+}
 
 void Load_data_worker::operator()() {
   mysqlsh::Mysql_thread t;
@@ -565,158 +665,198 @@ void Load_data_worker::operator()() {
     auto const conn_opts = m_opt.connection_options();
     session->connect(conn_opts);
   } catch (...) {
-    m_thread_exception[m_thread_id] = std::current_exception();
+    handle_exception();
     return;
   }
 
   execute(session, nullptr);
 }
 
+void Load_data_worker::init_session(
+    const std::shared_ptr<mysqlshdk::db::mysql::Session> &session,
+    const Import_table_options &options) {
+  const auto execute = [&session](const auto &sql) { session->execute(sql); };
+  const auto executef = [&session](const auto &sql, auto &&...args) {
+    session->executef(sql, std::forward<decltype(args)>(args)...);
+  };
+
+  // this sets the character_set_database and collation_database server
+  // variables to the values the schema has
+  executef("USE !;", options.schema());
+
+  // SQL mode:
+  //  - no_auto_value_on_zero - normally, 0 generates the next sequence
+  //    number, use this mode to prevent this behaviour (solves problems if
+  //    dump has 0 stored in an AUTO_INCREMENT column)
+  execute("SET SQL_MODE = 'no_auto_value_on_zero';");
+
+  // if user has specified the character set, set the session variables
+  // related to the client connection
+  if (!options.character_set().empty()) {
+    executef("SET NAMES ?;", options.character_set());
+  }
+
+  // BUG#34173126, BUG#33360787 - loading when global auto-commit is OFF fails
+  execute("SET autocommit = 1");
+  // set session variables
+  execute("SET unique_checks = 0");
+  execute("SET foreign_key_checks = 0");
+  execute("SET SESSION TRANSACTION ISOLATION LEVEL READ UNCOMMITTED");
+
+  try {
+    for (const auto &s : options.session_init_sql()) {
+      log_info("Executing custom session init SQL: %s", s.c_str());
+      execute(s);
+    }
+  } catch (const shcore::Error &e) {
+    throw shcore::Exception::runtime_error(
+        "Error while executing sessionInitSql: " + e.format());
+  }
+}
+
+std::string Load_data_worker::load_data_body(
+    const Import_table_options &options) {
+  const std::string on_duplicate_rows = [&options]() {
+    switch (options.duplicate_handling()) {
+      case Duplicate_handling::Default:
+        return "";
+
+      case Duplicate_handling::Replace:
+        return "REPLACE ";
+
+      case Duplicate_handling::Ignore:
+        return "IGNORE ";
+    }
+
+    throw std::logic_error("Unhandled Duplicate_handling value.");
+  }();
+
+  const std::string partition =
+      options.partition().empty()
+          ? ""
+          : "PARTITION (" + shcore::quote_identifier(options.partition()) +
+                ") ";
+
+  const std::string character_set =
+      options.character_set().empty()
+          ? ""
+          : "CHARACTER SET " +
+                shcore::quote_sql_string(options.character_set()) + " ";
+
+  const std::string compression = [&options]() {
+    using mysqlshdk::storage::Compression;
+
+    switch (options.compression()) {
+      case Compression::NONE:
+        return "";
+
+      case Compression::ZSTD:
+        return "COMPRESSION='ZSTD' ";
+
+      case Compression::GZIP:
+        throw std::logic_error("Unsupported LOAD DATA compression: gzip");
+    }
+
+    throw std::logic_error("Unhandled Compression value.");
+  }();
+
+  std::string query_body = on_duplicate_rows + "INTO TABLE " +
+                           shcore::quote_identifier(options.schema()) + '.' +
+                           shcore::quote_identifier(options.table()) + ' ' +
+                           partition + character_set + compression +
+                           options.dialect().build_sql();
+
+  const auto &decode_columns = options.decode_columns();
+
+  std::string query_columns;
+
+  if (const auto &columns = options.columns(); !columns.empty()) {
+    const auto placeholders =
+        shcore::str_join(columns, ", ", [&decode_columns](const auto &c) {
+          return std::visit(
+              [&decode_columns](auto &&value) {
+                using T = std::decay_t<decltype(value)>;
+
+                if constexpr (std::is_same_v<T, uint64_t>) {
+                  // user defined variable
+                  return "@" + std::to_string(value);
+                } else if constexpr (std::is_same_v<T, std::string>) {
+                  // column name
+                  std::string result;
+
+                  if (decode_columns.count(value)) {
+                    result += '@';
+                  }
+
+                  result += shcore::quote_identifier(value);
+
+                  return result;
+                } else {
+                  assert(false);
+                }
+              },
+              c);
+        });
+
+    query_columns = " (" + placeholders + ")";
+  }
+
+  if (!decode_columns.empty()) {
+    query_columns += " SET ";
+
+    for (const auto &it : decode_columns) {
+      if (it.second == "UNHEX" || it.second == "FROM_BASE64") {
+        query_columns +=
+            shcore::sqlformat("! = " + it.second + "(@!),", it.first, it.first);
+      } else if (!it.second.empty()) {
+        query_columns += shcore::sqlformat("! = ", it.first);
+        // Append "as is".
+        query_columns += "(" + it.second + "),";
+      }
+    }
+
+    query_columns.pop_back();  // strip the last ,
+  }
+
+  if (options.skip_rows_count()) {
+    query_body += " IGNORE ";
+    query_body += std::to_string(options.skip_rows_count());
+    query_body += " LINES";
+  }
+
+  query_body += query_columns;
+
+  return query_body;
+}
+
 void Load_data_worker::execute(
     const std::shared_ptr<mysqlshdk::db::mysql::Session> &session,
     std::unique_ptr<mysqlshdk::storage::IFile> file,
     const Transaction_options &options) {
+  const auto worker_name =
+      shcore::str_format("[Worker%03" PRId64 "]: ", m_thread_id);
+
   try {
     File_info fi;
     fi.worker_id = m_thread_id;
-    fi.prog_data_bytes = m_prog_sent_bytes;
+    fi.prog_data_bytes = m_prog_data_bytes;
     fi.prog_file_bytes = m_prog_file_bytes;
-    fi.user_interrupt = &m_interrupt;
+    fi.user_interrupt = m_interrupt;
     fi.max_rate = m_opt.max_rate();
+    fi.on_infile_read_end = [this]() { set_state(Thread_state::COMMITTING); };
     uint64_t max_trx_size = 0;
     const auto query = [&session](const auto &sql) {
       return session->query(sql);
     };
-    const auto execute = [&session](const auto &sql) { session->execute(sql); };
-    const auto executef = [&session](const auto &sql, auto &&... args) {
-      session->executef(sql, std::forward<decltype(args)>(args)...);
-    };
-
-    // this sets the character_set_database and collation_database server
-    // variables to the values the schema has
-    executef("USE !;", m_opt.schema());
-
-    // SQL mode:
-    //  - no_auto_value_on_zero - normally, 0 generates the next sequence
-    //    number, use this mode to prevent this behaviour (solves problems if
-    //    dump has 0 stored in an AUTO_INCREMENT column)
-    execute("SET SQL_MODE = 'no_auto_value_on_zero';");
-
-    // if user has specified the character set, set the session variables
-    // related to the client connection
-    if (!m_opt.character_set().empty()) {
-      executef("SET NAMES ?;", m_opt.character_set());
-    }
-
-    // BUG#34173126, BUG#33360787 - loading when global auto-commit is OFF fails
-    execute("SET autocommit = 1");
-    // set session variables
-    execute("SET unique_checks = 0");
-    execute("SET foreign_key_checks = 0");
-    execute("SET SESSION TRANSACTION ISOLATION LEVEL READ UNCOMMITTED");
 
+    init_session(session, m_opt);
     session->set_local_infile_userdata(static_cast<void *>(&fi));
     session->set_local_infile_init(local_infile_init);
     session->set_local_infile_read(local_infile_read);
     session->set_local_infile_end(local_infile_end);
     session->set_local_infile_error(local_infile_error);
 
-    const std::string on_duplicate_rows =
-        m_opt.replace_duplicates() ? std::string{"REPLACE "} : std::string{};
-
-    const std::string character_set =
-        m_opt.character_set().empty()
-            ? ""
-            : "CHARACTER SET " +
-                  shcore::quote_sql_string(m_opt.character_set()) + " ";
-
-    const std::string partition =
-        m_opt.partition().empty()
-            ? ""
-            : "PARTITION (" + shcore::quote_identifier(m_opt.partition()) +
-                  ") ";
-
-    try {
-      for (const auto &s : m_opt.session_init_sql()) {
-        log_info("Executing custom session init SQL: %s", s.c_str());
-        session->execute(s);
-      }
-    } catch (const shcore::Error &e) {
-      throw shcore::Exception::runtime_error(
-          "Error while executing sessionInitSql: " + e.format());
-    }
-
-    const std::string query_body =
-        on_duplicate_rows + "INTO TABLE " +
-        shcore::quote_identifier(m_opt.schema()) + '.' +
-        shcore::quote_identifier(m_opt.table()) + ' ' + partition +
-        character_set + m_opt.dialect().build_sql();
-
-    const auto &decode_columns = m_opt.decode_columns();
-
-    std::string query_ignore_lines;
-    std::string query_columns;
-    const auto columns = m_opt.columns().get();
-
-    if (columns && !columns->empty()) {
-      const std::vector<std::string> x(columns->size(), "!");
-      const auto placeholders = shcore::str_join(
-          *columns, ", ",
-          [&decode_columns](const shcore::Value &c) -> std::string {
-            if (c.type == shcore::Value_type::UInteger) {
-              // user defined variable
-              return "@" + c.as_string();
-            } else if (c.type == shcore::Value_type::Integer) {
-              // We do not want user variable to be negative: `@-1`
-              if (c.as_int() < 0) {
-                throw shcore::Exception::value_error(
-                    "User variable binding in 'columns' option must be "
-                    "non-negative integer value");
-              }
-              // user defined variable
-              return "@" + c.as_string();
-            } else if (c.type == shcore::Value_type::String) {
-              const auto column_name = c.as_string();
-              std::string prefix;
-
-              if (decode_columns.find(column_name) != decode_columns.end()) {
-                prefix = "@";
-              }
-
-              return prefix + shcore::quote_identifier(column_name);
-            } else {
-              throw shcore::Exception::type_error(
-                  "Option 'columns' " + type_name(shcore::Value_type::String) +
-                  " (column name) or non-negative " +
-                  type_name(shcore::Value_type::Integer) +
-                  " (user variable binding) expected, but value is " +
-                  type_name(c.type));
-            }
-          });
-
-      query_columns = " (" + placeholders + ")";
-    }
-
-    if (!decode_columns.empty()) {
-      query_columns += " SET ";
-
-      for (const auto &it : decode_columns) {
-        if (it.second == "UNHEX" || it.second == "FROM_BASE64") {
-          query_columns += shcore::sqlformat("! = " + it.second + "(@!),",
-                                             it.first, it.first);
-        } else if (!it.second.empty()) {
-          query_columns += shcore::sqlformat("! = ", it.first);
-          // Append "as is".
-          query_columns += "(" + it.second + "),";
-        }
-      }
-
-      query_columns.pop_back();  // strip the last ,
-    }
-
-    char worker_name[64];
-    snprintf(worker_name, sizeof(worker_name), "[Worker%03u] ",
-             static_cast<unsigned int>(m_thread_id));
+    const auto query_body = load_data_body(m_opt);
 
     uint64_t subchunk = 0;
     while (true) {
@@ -731,12 +871,12 @@ void Load_data_worker::execute(
       const auto format_error_message = [&worker_name, &fi, &r](
                                             const std::string &error,
                                             const std::string &extra = {}) {
-        const auto task = fi.filehandler ? fi.filehandler->filename() : "";
-        auto msg = worker_name + task + ": " + error;
+        const auto task =
+            fi.filehandler ? fi.filehandler->filename() + ": " : "";
+        auto msg = worker_name + task + error;
 
-        if (fi.range_read) {
-          msg += " @ file bytes range [" + std::to_string(r.range.first) +
-                 ", " + std::to_string(r.range.second) + ")";
+        if (!r.context.empty()) {
+          msg += r.context;
         }
 
         if (!extra.empty()) {
@@ -755,29 +895,18 @@ void Load_data_worker::execute(
               break;
             }
 
-            fi.filehandler = m_opt.create_file_handle(r.file_path);
+            fi.filehandler = std::move(r.file);
             fi.range_read = r.range_read;
 
             if (r.range_read) {
               fi.bytes_left = r.range.second - r.range.first;
-              // TODO(pawel): maxBytesPerTransaction should not be ignored in
-              // case of a single uncompressed file imported in chunks
+              // maxBytesPerTransaction is ignored in case of a file imported in
+              // chunks, because we assume that in this case bytesPerChunk ==
+              // maxBytesPerTransaction
               max_trx_size = 0;
-
-              // if fi.range_read == true, we're importing in chunks from a
-              // single uncompressed file, rows were already skipped
-              query_ignore_lines.clear();
             } else {
               fi.bytes_left = 0;
               max_trx_size = m_opt.max_transaction_size();
-
-              if (m_opt.skip_rows_count() > 0) {
-                // this handles the case when a single compressed file or
-                // multiple files are being imported and skipRows is set
-                query_ignore_lines = " IGNORE " +
-                                     std::to_string(m_opt.skip_rows_count()) +
-                                     " LINES";
-              }
             }
 
             fi.buffer =
@@ -795,7 +924,7 @@ void Load_data_worker::execute(
           fi.bytes_left = 0;
         }
       } catch (const std::exception &e) {
-        m_thread_exception[m_thread_id] = std::current_exception();
+        handle_exception();
         mysqlsh::current_console()->print_error(format_error_message(e.what()));
         throw std::exception(e);
       }
@@ -804,11 +933,10 @@ void Load_data_worker::execute(
       std::shared_ptr<mysqlshdk::db::IResult> load_result = nullptr;
       const std::string query_prefix = shcore::sqlformat(
           "LOAD DATA LOCAL INFILE ? ", fi.filehandler->full_path().masked());
-      const std::string full_query =
-          query_prefix + query_body + query_ignore_lines + query_columns;
+      const std::string full_query = query_prefix + query_body;
 
 #ifndef NDEBUG
-      log_debug("%s %s %i", worker_name, full_query.c_str(),
+      log_debug("%s %s %i", worker_name.c_str(), full_query.c_str(),
                 m_range_queue != nullptr);
 #endif
 
@@ -821,18 +949,20 @@ void Load_data_worker::execute(
       });
 
       try {
+        set_state(Thread_state::READING);
         fi.buffer.before_query();
         load_result = query(m_query_comment + full_query);
+        set_state(Thread_state::IDLE);
         fi.buffer.flush_done(&fi.continuation);
-        m_stats.total_data_bytes += fi.data_bytes;
-        m_stats.total_file_bytes += fi.file_bytes;
+        m_stats->total_data_bytes += fi.data_bytes;
+        m_stats->total_file_bytes += fi.file_bytes;
 
         if (!fi.continuation) {
           // increase the counter only when there are no more subchunks
-          ++m_stats.total_files_processed;
+          ++m_stats->total_files_processed;
         }
       } catch (const mysqlshdk::db::Error &e) {
-        m_thread_exception[m_thread_id] = std::current_exception();
+        handle_exception();
         const auto error_msg = format_error_message(e.format(), full_query);
         mysqlsh::current_console()->print_error(error_msg);
 
@@ -845,12 +975,12 @@ void Load_data_worker::execute(
 
         throw std::runtime_error(error_msg);
       } catch (const mysqlshdk::rest::Connection_error &e) {
-        m_thread_exception[m_thread_id] = std::current_exception();
+        handle_exception();
         const auto error_msg = format_error_message(e.what());
         mysqlsh::current_console()->print_error(error_msg);
         throw std::runtime_error(error_msg);
       } catch (const std::exception &e) {
-        m_thread_exception[m_thread_id] = std::current_exception();
+        handle_exception();
         const auto error_msg = format_error_message(e.what());
         mysqlsh::current_console()->print_error(error_msg);
         throw std::exception(e);
@@ -885,10 +1015,10 @@ void Load_data_worker::execute(
           sscanf(mysql_info,
                  "Records: %zu  Deleted: %zu  Skipped: %zu  Warnings: %zu\n",
                  &records, &deleted, &skipped, &warnings);
-          m_stats.total_records += records;
-          m_stats.total_deleted += deleted;
-          m_stats.total_skipped += skipped;
-          m_stats.total_warnings += warnings;
+          m_stats->total_records += records;
+          m_stats->total_deleted += deleted;
+          m_stats->total_skipped += skipped;
+          m_stats->total_warnings += warnings;
         }
 
         if (warnings_num > 0) {
@@ -945,8 +1075,26 @@ void Load_data_worker::execute(
 
       if (!m_range_queue && !fi.continuation) break;
     }
+  } catch (const std::exception &e) {
+    if (!*m_thread_exception) {
+      mysqlsh::current_console()->print_error(worker_name + e.what());
+      handle_exception();
+    }
   } catch (...) {
-    m_thread_exception[m_thread_id] = std::current_exception();
+    handle_exception();
+  }
+}
+
+void Load_data_worker::handle_exception() {
+  set_state(Thread_state::ERROR);
+  *m_thread_exception = std::current_exception();
+}
+
+void Load_data_worker::set_state(Thread_state new_state) {
+  if (m_state != new_state) {
+    --m_stats->thread_states[m_state];
+    m_state = new_state;
+    ++m_stats->thread_states[m_state];
   }
 }
 
