@@ -27,6 +27,7 @@
 #define MYSQLSHDK_LIBS_CONFIG_CONFIG_FILE_HANDLER_H_
 
 #include <memory>
+#include <optional>
 #include <string>
 
 #include "mysqlshdk/libs/config/config.h"
@@ -143,7 +144,7 @@ class Config_file_handler : public IConfig_handler {
    *        std::runtime_error if any runtime exception happens while writing
    *        to the option file(s).
    */
-  void apply() override;
+  void apply(bool skip_default_value_check = false) override;
 
   std::string get_server_uuid() const override { return m_server_uuid; }
 
