@@ -23,23 +23,20 @@
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include <mysqld_error.h>
+#include "mysqlshdk/libs/mysql/instance.h"
 
-#include <algorithm>
 #include <array>
 #include <map>
 #include <string_view>
-#include <utility>
 
-#include "mysqlshdk/libs/mysql/instance.h"
+#include <mysqld_error.h>
+
 #include "mysqlshdk/libs/utils/logger.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_net.h"
 #include "mysqlshdk/libs/utils/utils_sqlstring.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
-#include "scripting/types.h"
-
 namespace mysqlshdk {
 namespace mysql {
 
@@ -66,9 +63,6 @@ void Auth_options::set(mysqlshdk::db::Connection_options *copts) const {
   copts->set_ssl_options(ssl_options);
 }
 
-Instance::Instance(const std::shared_ptr<db::ISession> &session)
-    : _session(session) {}
-
 void Instance::register_warnings_callback(const Warnings_callback &callback) {
   m_warnings_callback = callback;
 }
@@ -418,28 +412,28 @@ bool Instance::is_ssl_enabled() const {
  * performance_schema variables_info table.
  */
 
-bool Instance::has_variable_compiled_value(const std::string &name) const {
-  bool perf_schema_on = is_performance_schema_enabled();
-  if (!perf_schema_on)
-    throw std::runtime_error("Unable to check if variable '" + name +
-                             "' has the default (compiled) value since "
-                             "performance_schema is not enabled.");
-  std::string variable_default_stmt_fmt =
-      "SELECT variable_source "
-      "FROM performance_schema.variables_info "
-      "WHERE variable_name = ?";
-  shcore::sqlstring variable_default_stmt =
-      shcore::sqlstring(variable_default_stmt_fmt, 0);
-  variable_default_stmt << name;
-  variable_default_stmt.done();
-  auto resultset = query(variable_default_stmt);
-  auto row = resultset->fetch_one();
-  if (row)
-    return row->get_string(0) == "COMPILED";
-  else
-    throw std::runtime_error(
-        "Unable to find variable '" + name +
-        "' in the performance_schema.variables_info table.");
+bool Instance::has_variable_compiled_value(std::string_view name) const {
+  if (!is_performance_schema_enabled())
+    throw std::runtime_error(shcore::str_format(
+        "Unable to check if variable '%.*s' has the default (compiled) value "
+        "since performance_schema is not enabled.",
+        static_cast<int>(name.size()), name.data()));
+
+  auto stmt =
+      "SELECT (variable_source = 'COMPILED') FROM "
+      "performance_schema.variables_info WHERE variable_name = ?"_sql;
+  stmt << name;
+  stmt.done();
+
+  auto resultset = query(stmt);
+  if (auto row = resultset->fetch_one(); row) {
+    return (row->get_int(0) != 0);
+  }
+
+  throw std::runtime_error(
+      shcore::str_format("Unable to find variable '%.*s' in the "
+                         "performance_schema.variables_info table.",
+                         static_cast<int>(name.size()), name.data()));
 }
 
 /**
@@ -626,8 +620,8 @@ bool Instance::is_read_only(bool super) const {
 }
 
 utils::Version Instance::get_version() const {
-  if (_version == utils::Version()) _version = _session->get_server_version();
-  return _version;
+  if (!m_version) m_version = m_session->get_server_version();
+  return m_version;
 }
 
 /**
@@ -672,7 +666,7 @@ bool Instance::user_exists(const std::string &username,
     // the current_account doesn't have enough privileges to execute the query
     if (err.code() == ER_TABLEACCESS_DENIED_ERROR) {
       // Get the current username and hostname
-      // https://dev.mysql.com/doc/refman/5.7/en/information-functions.html
+      // https://dev.mysql.com/doc/refman/en/information-functions.html
       // #function_current-user
       std::string current_user, current_host;
       get_current_user(&current_user, &current_host);
@@ -720,7 +714,7 @@ std::optional<bool> Instance::is_set_persist_supported() const {
 }
 
 std::optional<std::string> Instance::get_persisted_value(
-    const std::string &variable_name) const {
+    std::string_view variable_name) const {
   auto result = queryf(
       "SELECT variable_value "
       "FROM performance_schema.persisted_variables "
@@ -734,8 +728,6 @@ std::optional<std::string> Instance::get_persisted_value(
 }
 
 std::vector<std::string> Instance::get_fence_sysvars() const {
-  std::vector<std::string> result;
-
   // Create the query to get the fence sysvars.
   std::string str_query = "SELECT";
   for (auto it = k_fence_sysvars.cbegin(); it != k_fence_sysvars.cend(); ++it) {
@@ -744,19 +736,22 @@ std::vector<std::string> Instance::get_fence_sysvars() const {
   }
 
   // Execute the query and add all the enabled sysvars to the result list.
-  auto resultset = _session->query(str_query);
+  auto resultset = m_session->query(str_query);
   auto row = resultset->fetch_one();
-  if (row) {
-    for (size_t i = 0; i < k_fence_sysvars.size(); ++i) {
-      if (row->get_int(i) != 0) {
-        result.push_back(std::string{k_fence_sysvars.at(i)});
-      }
-    }
-    return result;
-  } else {
+  if (!row)
     throw std::logic_error(
         "No result return from query for get_fence_sysvars()");
+
+  std::vector<std::string> result;
+
+  result.reserve(k_fence_sysvars.size());
+  for (size_t i = 0; i < k_fence_sysvars.size(); ++i) {
+    if (row->get_int(i) != 0) {
+      result.push_back(std::string{k_fence_sysvars.at(i)});
+    }
   }
+
+  return result;
 }
 
 void Instance::suppress_binary_log(bool flag) {
@@ -796,14 +791,13 @@ void Instance::execute(const std::string &sql) const {
   }
 }
 
-void Instance::process_result_warnings(const std::string &sql,
+void Instance::process_result_warnings(std::string_view sql,
                                        mysqlshdk::db::IResult &result) const {
-  // Call the Warnings_callback if registered
   if (!m_warnings_callback) return;
 
   // Get all the Warnings
   while (auto warning = result.fetch_one_warning()) {
-    std::string warning_level;
+    std::string_view warning_level;
 
     switch (warning->level) {
       case db::Warning::Level::Note:
@@ -818,10 +812,14 @@ void Instance::process_result_warnings(const std::string &sql,
     }
 
     m_warnings_callback(sql, warning->code, warning_level, warning->msg);
-
-    warning = result.fetch_one_warning();
   }
 }
 
+std::string Instance::generate_uuid() const {
+  auto resultset = query("SELECT UUID()");
+  auto row = resultset->fetch_one();
+  return row->get_string(0);
+}
+
 }  // namespace mysql
 }  // namespace mysqlshdk
