@@ -28,17 +28,16 @@
 #ifndef MYSQLSHDK_LIBS_MYSQL_MYCNF_H_
 #define MYSQLSHDK_LIBS_MYSQL_MYCNF_H_
 
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "mysqlshdk/libs/utils/nullable.h"
-
 namespace mysqlshdk {
 namespace mysql {
 namespace mycnf {
 
-using Option = std::pair<std::string, utils::nullable<std::string>>;
+using Option = std::pair<std::string, std::optional<std::string>>;
 
 void update_options(const std::string &path, const std::string &section,
                     const std::vector<Option> &mycnf_options);
