@@ -26,11 +26,13 @@
 #include "mysqlshdk/libs/mysql/replication.h"
 
 #include <algorithm>
+#include <array>
 #include <random>
 #include <string>
 #include <vector>
 
-#include "modules/adminapi/common/common.h"
+#include "modules/adminapi/common/server_features.h"
+#include "modules/errors.h"
 #include "mysqlshdk/libs/mysql/instance.h"
 #include "mysqlshdk/libs/utils/structured_text.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
@@ -124,55 +126,55 @@ std::string to_string(Replication_channel::Applier::Status status) {
 }
 
 std::string to_string(const Replication_channel::Error &error) {
-  if (error.code != 0) {
-    return shcore::str_format("%s (%i) at %s", error.message.c_str(),
-                              error.code, error.timestamp.c_str());
-  }
-  return "";
+  if (error.code == 0) return {};
+  return shcore::str_format("%s (%i) at %s", error.message.c_str(), error.code,
+                            error.timestamp.c_str());
 }
 
 std::string format_status(const Replication_channel &channel, bool verbose) {
   // This follows the KVP format
 
   std::string msg;
+  msg.reserve(128);  // better than nothing
 
   msg += shcore::make_kvp(
-      "source", channel.host + ":" + std::to_string(channel.port), '"');
-  msg += " " + shcore::make_kvp("channel", channel.channel_name);
-  msg += " " + shcore::make_kvp("status", to_string(channel.status()));
+      "source", shcore::str_format("%s:%d", channel.host.c_str(), channel.port),
+      '"');
+  msg.append(" ").append(shcore::make_kvp("channel", channel.channel_name));
+  msg.append(" ").append(
+      shcore::make_kvp("status", to_string(channel.status())));
 
-  if (verbose || channel.status() != Replication_channel::OFF) {
-    msg +=
-        " " + shcore::make_kvp("receiver", to_string(channel.receiver.state));
-    if (channel.receiver.last_error.code != 0)
-      msg +=
-          " " + shcore::make_kvp("last_error",
-                                 to_string(channel.receiver.last_error), '"');
+  if (!verbose && channel.status() == Replication_channel::OFF) return msg;
 
-    if (channel.coordinator.state != Replication_channel::Coordinator::NONE) {
-      msg += " " + shcore::make_kvp("coordinator",
-                                    to_string(channel.coordinator.state));
-      if (channel.coordinator.last_error.code != 0)
-        msg += " " + shcore::make_kvp("last_error",
-                                      to_string(channel.coordinator.last_error),
-                                      '"');
-    }
-    if (channel.appliers.size() == 1) {
-      msg += " " + shcore::make_kvp("applier",
-                                    to_string(channel.appliers.front().state));
-      if (channel.appliers.front().last_error.code != 0)
-        msg += " " + shcore::make_kvp(
-                         "last_error",
-                         to_string(channel.appliers.front().last_error), '"');
-    } else {
-      int i = 0;
-      for (const auto &a : channel.appliers) {
-        msg += " " + shcore::make_kvp(("applier" + std::to_string(i)).c_str(),
-                                      to_string(a.state));
-        if (a.last_error.code != 0)
-          msg += " " + shcore::make_kvp("last_error", to_string(a.last_error));
-        ++i;
-      }
+  msg += " " + shcore::make_kvp("receiver", to_string(channel.receiver.state));
+  if (channel.receiver.last_error.code != 0)
+    msg += " " + shcore::make_kvp("last_error",
+                                  to_string(channel.receiver.last_error), '"');
+
+  if (channel.coordinator.state != Replication_channel::Coordinator::NONE) {
+    msg += " " + shcore::make_kvp("coordinator",
+                                  to_string(channel.coordinator.state));
+    if (channel.coordinator.last_error.code != 0)
+      msg += " " + shcore::make_kvp("last_error",
+                                    to_string(channel.coordinator.last_error),
+                                    '"');
+  }
+
+  if (channel.appliers.size() == 1) {
+    msg += " " + shcore::make_kvp("applier",
+                                  to_string(channel.appliers.front().state));
+    if (channel.appliers.front().last_error.code != 0)
+      msg += " " + shcore::make_kvp(
+                       "last_error",
+                       to_string(channel.appliers.front().last_error), '"');
+  } else {
+    int i = 0;
+    for (const auto &a : channel.appliers) {
+      msg += " " + shcore::make_kvp(("applier" + std::to_string(i)).c_str(),
+                                    to_string(a.state));
+      if (a.last_error.code != 0)
+        msg += " " + shcore::make_kvp("last_error", to_string(a.last_error));
+      ++i;
     }
   }
 
@@ -259,7 +261,7 @@ void unserialize_channel_applier_info(const mysqlshdk::db::Row_ref_by_name &row,
   if (row.is_null("w_state")) return;
 
   Replication_channel::Applier applier;
-  if (std::string state = row.get_string("w_state"); state == "ON")
+  if (auto state = row.get_string("w_state"); state == "ON")
     applier.state = Replication_channel::Applier::ON;
   else if (state == "OFF")
     applier.state = Replication_channel::Applier::OFF;
@@ -367,7 +369,7 @@ std::vector<Replication_channel> get_incoming_channels(
       }
       unserialize_channel_applier_info(row, &channel);
     }
-    channels.push_back(channel);
+    channels.push_back(std::move(channel));
   }
 
   return channels;
@@ -503,128 +505,188 @@ bool get_channel_state(const mysqlshdk::mysql::IInstance &instance,
 }
 
 namespace {
-void unserialize_channel_master_info(
-    const mysqlshdk::db::Row_ref_by_name &row,
-    Replication_channel_master_info *out_master_info) {
-  out_master_info->master_log_name = row.get_string("Master_log_name");
-  out_master_info->master_log_pos = row.get_uint("Master_log_pos");
-  out_master_info->host = row.get_string("Host");
-  out_master_info->user_name = row.get_string("User_name");
-  out_master_info->port = row.get_int("Port");
-  out_master_info->connect_retry = row.get_uint("Connect_retry");
-  out_master_info->enabled_ssl = row.get_int("Enabled_ssl");
-  out_master_info->ssl_ca = row.get_string("Ssl_ca");
-  out_master_info->ssl_capath = row.get_string("Ssl_capath");
-  out_master_info->ssl_cert = row.get_string("Ssl_cert");
-  out_master_info->ssl_cipher = row.get_string("Ssl_cipher");
-  out_master_info->ssl_key = row.get_string("Ssl_key");
-  out_master_info->ssl_verify_server_cert =
-      row.get_int("Ssl_verify_server_cert");
-  out_master_info->heartbeat = row.get_double("Heartbeat");
-  out_master_info->bind = row.get_string("Bind");
-  out_master_info->ignored_server_ids = row.get_string("Ignored_server_ids");
-  out_master_info->uuid = row.get_string("Uuid");
-  out_master_info->retry_count = row.get_uint("Retry_count");
-  out_master_info->ssl_crl = row.get_string("Ssl_crl");
-  out_master_info->ssl_crlpath = row.get_string("Ssl_crlpath");
-  out_master_info->enabled_auto_position = row.get_int("Enabled_auto_position");
-  out_master_info->channel_name = row.get_string("Channel_name");
-  out_master_info->tls_version = row.get_string("Tls_version");
 
-  if (row.has_field("Public_key_path") && !row.is_null("Public_key_path"))
-    out_master_info->public_key_path = row.get_string("Public_key_path");
-  if (row.has_field("Get_public_key") && !row.is_null("Get_public_key"))
-    out_master_info->get_public_key = row.get_int("Get_public_key");
-  if (row.has_field("Network_namespace") && !row.is_null("Network_namespace"))
-    out_master_info->network_namespace = row.get_string("Network_namespace");
-  if (row.has_field("Master_compression_algorithm") &&
-      !row.is_null("Master_compression_algorithm"))
-    out_master_info->master_compression_algorithm =
-        row.get_string("Master_compression_algorithm");
-  if (row.has_field("Master_zstd_compression_level") &&
-      !row.is_null("Master_zstd_compression_level"))
-    out_master_info->master_zstd_compression_level =
-        row.get_uint("Master_zstd_compression_level");
-  if (row.has_field("Tls_ciphersuites") && !row.is_null("Tls_ciphersuites"))
-    out_master_info->tls_ciphersuites = row.get_string("Tls_ciphersuites");
+Replication_channel_master_info unserialize_channel_master_info(
+    const mysqlshdk::db::Row_ref_by_name &row, bool from_perf_schema) {
+  Replication_channel_master_info minfo;
+
+  minfo.host = row.get_string("host");
+  minfo.user_name = row.get_string(from_perf_schema ? "user" : "user_name");
+  minfo.port = row.get_int("port");
+  minfo.connect_retry = row.get_uint(
+      from_perf_schema ? "connection_retry_interval" : "connect_retry");
+
+  if (from_perf_schema) {
+    auto value = row.get_string("ssl_allowed");
+    minfo.enabled_ssl = shcore::str_caseeq(value, "yes");
+  } else {
+    minfo.enabled_ssl = static_cast<int>(row.get_int("enabled_ssl"));
+  }
+
+  minfo.ssl_ca = row.get_string(from_perf_schema ? "ssl_ca_file" : "ssl_ca");
+  minfo.ssl_capath =
+      row.get_string(from_perf_schema ? "ssl_ca_path" : "ssl_capath");
+  minfo.ssl_cert =
+      row.get_string(from_perf_schema ? "ssl_certificate" : "ssl_cert");
+  minfo.ssl_cipher = row.get_string("ssl_cipher");
+  minfo.ssl_key = row.get_string("ssl_key");
+
+  if (from_perf_schema) {
+    auto value = row.get_string("ssl_verify_server_certificate");
+    minfo.ssl_verify_server_cert = shcore::str_caseeq(value, "yes");
+  } else {
+    minfo.ssl_verify_server_cert = row.get_int("ssl_verify_server_cert");
+  }
+
+  minfo.heartbeat_period =
+      row.get_double(from_perf_schema ? "heartbeat_interval" : "heartbeat");
+  minfo.bind = row.get_string(from_perf_schema ? "network_interface" : "bind");
+  minfo.retry_count =
+      row.get_uint(from_perf_schema ? "connection_retry_count" : "retry_count");
+  minfo.ssl_crl = row.get_string(from_perf_schema ? "ssl_crl_file" : "ssl_crl");
+  minfo.ssl_crlpath =
+      row.get_string(from_perf_schema ? "ssl_crl_path" : "ssl_crlpath");
+
+  if (from_perf_schema) {
+    auto value = row.get_string("auto_position");
+    minfo.enabled_auto_position = !shcore::str_caseeq(value, "0");
+  } else {
+    minfo.enabled_auto_position = row.get_int("enabled_auto_position");
+  }
+
+  minfo.channel_name = row.get_string("channel_name");
+  minfo.tls_version = row.get_string("tls_version");
+
+  if (row.has_field("public_key_path") && !row.is_null("public_key_path"))
+    minfo.public_key_path = row.get_string("public_key_path");
+  if (row.has_field("network_namespace") && !row.is_null("network_namespace"))
+    minfo.network_namespace = row.get_string("network_namespace");
+  {
+    auto column_name = from_perf_schema ? "compression_algorithm"
+                                        : "master_compression_algorithm";
+    if (row.has_field(column_name) && !row.is_null(column_name))
+      minfo.compression_algorithm = row.get_string(column_name);
+  }
+  {
+    auto column_name = from_perf_schema ? "zstd_compression_level"
+                                        : "master_zstd_compression_level";
+    if (row.has_field(column_name) && !row.is_null(column_name))
+      minfo.zstd_compression_level = row.get_uint(column_name);
+  }
+  if (row.has_field("tls_ciphersuites") && !row.is_null("tls_ciphersuites"))
+    minfo.tls_ciphersuites = row.get_string("tls_ciphersuites");
+
+  return minfo;
 }
 
-void unserialize_channel_relay_log_info(
-    const mysqlshdk::db::Row_ref_by_name &row,
-    Replication_channel_relay_log_info *out_relay_log_info) {
-  out_relay_log_info->relay_log_name = row.get_string("Relay_log_name", "");
-  out_relay_log_info->relay_log_pos = row.get_uint("Relay_log_pos", 0);
-  out_relay_log_info->master_log_name = row.get_string("Master_log_name");
-  out_relay_log_info->master_log_pos = row.get_uint("Master_log_pos");
-  out_relay_log_info->sql_delay = row.get_uint("Sql_delay");
-  out_relay_log_info->number_of_workers = row.get_uint("Number_of_workers");
-  out_relay_log_info->id = row.get_uint("Id");
-  out_relay_log_info->channel_name = row.get_string("Channel_name");
-  if (row.has_field("Privilege_checks_username") &&
-      !row.is_null("Privilege_checks_username"))
-    out_relay_log_info->privilege_checks_username =
-        row.get_string("Privilege_checks_username");
-  if (row.has_field("Privilege_checks_hostname") &&
-      !row.is_null("Privilege_checks_hostname"))
-    out_relay_log_info->privilege_checks_hostname =
-        row.get_string("Privilege_checks_hostname");
-  if (row.has_field("Require_row_format") && !row.is_null("Require_row_format"))
-    out_relay_log_info->require_row_format = row.get_int("Require_row_format");
+Replication_channel_relay_log_info unserialize_channel_relay_log_info(
+    const mysqlshdk::db::Row_ref_by_name &row, bool from_perf_schema) {
+  Replication_channel_relay_log_info rinfo;
+
+  rinfo.sql_delay =
+      row.get_uint(from_perf_schema ? "desired_delay" : "Sql_delay");
+  rinfo.number_of_workers = row.get_uint("number_of_workers");
+  rinfo.channel_name = row.get_string("channel_name");
+
+  {
+    auto column_name = from_perf_schema ? "privilege_checks_user"
+                                        : "privilege_checks_username";
+
+    if (row.has_field(column_name) && !row.is_null(column_name))
+      rinfo.privilege_checks_user = row.get_string(column_name);
+  }
+
+  return rinfo;
 }
 
 }  // namespace
 
 bool get_channel_info(const mysqlshdk::mysql::IInstance &instance,
-                      const std::string &channel_name,
+                      std::string_view channel_name,
                       Replication_channel_master_info *out_master_info,
                       Replication_channel_relay_log_info *out_relay_log_info) {
-  assert(out_master_info && out_relay_log_info);
+  if (!out_master_info && !out_relay_log_info) return false;
 
-  {
-    auto result = instance.queryf(
-        "SELECT * FROM mysql.slave_master_info"
-        " WHERE channel_name = ?",
-        channel_name);
-    if (auto row = result->fetch_one_named()) {
-      unserialize_channel_master_info(row, out_master_info);
+  auto from_perf_schema =
+      instance.get_version() >= k_perf_schema_channels_min_version;
+
+  if (out_master_info) {
+    std::string query;
+    if (!from_perf_schema)
+      query = "SELECT * FROM mysql.slave_master_info WHERE channel_name = ?";
+    else
+      query =
+          "SELECT * FROM "
+          "performance_schema.replication_connection_configuration WHERE "
+          "channel_name = ?";
+
+    auto result = instance.queryf(query, channel_name);
+    if (auto row = result->fetch_one_named(); row) {
+      *out_master_info = unserialize_channel_master_info(row, from_perf_schema);
     } else {
       return false;
     }
   }
 
-  {
-    auto result = instance.queryf(
-        "SELECT * FROM mysql.slave_relay_log_info"
-        " WHERE channel_name = ?",
-        channel_name);
-    if (auto row = result->fetch_one_named()) {
-      unserialize_channel_relay_log_info(row, out_relay_log_info);
+  if (out_relay_log_info) {
+    std::string query;
+    if (!from_perf_schema)
+      query = "SELECT * FROM mysql.slave_relay_log_info WHERE channel_name = ?";
+    else
+      query =
+          "SELECT tconfig.*, tworker.number_of_workers FROM "
+          "performance_schema.replication_applier_configuration tconfig INNER "
+          "JOIN (SELECT channel_name, count(*) AS number_of_workers FROM "
+          "performance_schema.replication_applier_status_by_worker GROUP BY 1) "
+          "tworker ON (tconfig.channel_name = tworker.channel_name) WHERE "
+          "(tconfig.channel_name = ?);";
+
+    auto result = instance.queryf(query, channel_name);
+    if (auto row = result->fetch_one_named(); row) {
+      *out_relay_log_info =
+          unserialize_channel_relay_log_info(row, from_perf_schema);
     }
   }
 
   return true;
 }
 
-std::vector<Slave_host> get_slaves(
-    const mysqlshdk::mysql::IInstance &instance) {
-  std::vector<Slave_host> slaves;
+bool has_replicas(const mysqlshdk::mysql::IInstance &instance) {
+  std::shared_ptr<mysqlshdk::db::IResult> result;
+  if (instance.get_version() < utils::Version(8, 2, 0)) {
+    result = instance.query("SHOW SLAVE HOSTS");
+  } else {
+    result = instance.query("SHOW REPLICAS");
+  }
+  return static_cast<bool>(result->fetch_one_named());
+}
 
-  auto result = instance.query("SHOW SLAVE HOSTS");
-  while (auto row = result->fetch_one_named()) {
-    Slave_host host;
-    host.host = row.get_string("Host");
-    host.port = row.get_uint("Port");
-    host.uuid = row.get_string("Slave_UUID");
-    slaves.push_back(host);
+std::vector<Slave_host> get_replicas(
+    const mysqlshdk::mysql::IInstance &instance) {
+  std::vector<Slave_host> replicas;
+
+  {
+    auto use_new_term = instance.get_version() >= utils::Version(8, 2, 0);
+
+    auto result =
+        instance.query(use_new_term ? "SHOW REPLICAS" : "SHOW SLAVE HOSTS");
+
+    while (auto row = result->fetch_one_named()) {
+      Slave_host host;
+      host.host = row.get_string("Host");
+      host.port = row.get_uint("Port");
+      host.uuid = row.get_string(use_new_term ? "Replica_UUID" : "Slave_UUID");
+      replicas.push_back(std::move(host));
+    }
   }
 
-  // Sort by host/name to ensure deterministic output
-  std::sort(slaves.begin(), slaves.end(),
+  // Sort by host/port to ensure deterministic output
+  std::sort(replicas.begin(), replicas.end(),
             [](const Slave_host &a, const Slave_host &b) {
               return a.host < b.host || (a.host == b.host && a.port < b.port);
             });
 
-  return slaves;
+  return replicas;
 }
 
 Replication_channel::Status Replication_channel::status() const {
@@ -712,14 +774,21 @@ Replication_channel::Status Replication_channel::applier_status() const {
 
 std::string get_replication_user(const mysqlshdk::mysql::IInstance &instance,
                                  const std::string &channel_name) {
-  return instance.queryf_one_string(
-      0, "",
-      "SELECT User_name FROM mysql.slave_master_info WHERE Channel_name = ?",
-      channel_name);
+  if (instance.get_version() < k_perf_schema_channels_min_version)
+    return instance.queryf_one_string(
+        0, "",
+        "SELECT User_name FROM mysql.slave_master_info WHERE Channel_name = ?",
+        channel_name);
+
+  return instance.queryf_one_string(0, "",
+                                    "SELECT user FROM "
+                                    "performance_schema.replication_connection_"
+                                    "configuration WHERE channel_name = ?",
+                                    channel_name);
 }
 
 Replication_channel wait_replication_done_connecting(
-    const mysqlshdk::mysql::IInstance &slave, const std::string &channel_name) {
+    const mysqlshdk::mysql::IInstance &slave, std::string_view channel_name) {
   // Check that the IO thread for the channel switches to ON/OFF.
   // NOTE: It is assumed that the IO thread will not stay forever with the
   //       state CONNECTING and will eventually switch to ON or OFF (stopped).
@@ -776,19 +845,21 @@ size_t estimate_gtid_set_size(const std::string &gtid_set) {
   size_t count = 0;
   shcore::str_itersplit(
       gtid_set,
-      [&count](const std::string &s) {
+      [&count](std::string_view s) {
         size_t p = s.find(':');
-        if (p != std::string::npos) {
-          size_t begin, end;
-          switch (sscanf(&s[p + 1], "%zu-%zu", &begin, &end)) {
-            case 2:
-              count += end - begin + 1;
-              break;
-            case 1:
-              count++;
-              break;
-          }
+        if (p == std::string::npos) return true;
+
+        std::string range{s};
+        size_t begin, end;
+        switch (sscanf(&range[p + 1], "%zu-%zu", &begin, &end)) {
+          case 2:
+            count += end - begin + 1;
+            break;
+          case 1:
+            count++;
+            break;
         }
+
         return true;
       },
       ",");
@@ -865,34 +936,32 @@ Gtid_set_relation compare_gtid_sets(const mysqlshdk::mysql::IInstance &server,
     return Gtid_set_relation::CONTAINS;
   }
 
-  // Set some session tempvars for caching
-  server.executef("SET @gtidset_a=?", gtidset_a);
-  server.executef("SET @gtidset_b=?", gtidset_b);
+  auto set_a = mysqlshdk::mysql::Gtid_set::from_normalized_string(
+      std::string{gtidset_a});
+  auto set_b = mysqlshdk::mysql::Gtid_set::from_normalized_string(
+      std::string{gtidset_b});
 
-  std::string a_sub_b = server.queryf_one_string(
-      0, "", "SELECT GTID_SUBTRACT(@gtidset_a, @gtidset_b)");
-  std::string b_sub_a = server.queryf_one_string(
-      0, "", "SELECT GTID_SUBTRACT(@gtidset_b, @gtidset_a)");
-
-  if (out_missing_from_a) *out_missing_from_a = b_sub_a;
-  if (out_missing_from_b) *out_missing_from_b = a_sub_b;
-
-  if (a_sub_b.empty() && b_sub_a.empty()) {
-    return Gtid_set_relation::EQUAL;
-  } else if (a_sub_b.empty() && !b_sub_a.empty()) {
-    return Gtid_set_relation::CONTAINED;
-  } else if (!a_sub_b.empty() && b_sub_a.empty()) {
-    return Gtid_set_relation::CONTAINS;
-  } else {
-    std::string ab_intersection =
-        server.queryf_one_string(0, "",
-                                 "SELECT GTID_SUBTRACT(@gtidset_a, "
-                                 "GTID_SUBTRACT(@gtidset_a, @gtidset_b))");
-    if (ab_intersection.empty())
-      return Gtid_set_relation::DISJOINT;
-    else
-      return Gtid_set_relation::INTERSECTS;
+  if (!mysqlsh::dba::supports_gtid_tags(server.get_version()) &&
+      (set_a.has_tags() || set_b.has_tags())) {
+    throw shcore::Exception(
+        shcore::str_format("Instance '%s' doesn't support GTID tags.",
+                           server.descr().c_str()),
+        SHERR_UNSUPPORTED_GTID_TAG);
   }
+
+  auto a_sub_b = mysqlshdk::mysql::Gtid_set{set_a}.subtract(set_b, server);
+  auto b_sub_a = mysqlshdk::mysql::Gtid_set{set_b}.subtract(set_a, server);
+
+  if (out_missing_from_a) *out_missing_from_a = b_sub_a.str();
+  if (out_missing_from_b) *out_missing_from_b = a_sub_b.str();
+
+  if (a_sub_b.empty() && b_sub_a.empty()) return Gtid_set_relation::EQUAL;
+  if (a_sub_b.empty() && !b_sub_a.empty()) return Gtid_set_relation::CONTAINED;
+  if (!a_sub_b.empty() && b_sub_a.empty()) return Gtid_set_relation::CONTAINS;
+
+  set_b.intersect(set_a, server);
+  return set_b.empty() ? Gtid_set_relation::DISJOINT
+                       : Gtid_set_relation::INTERSECTS;
 }
 
 std::string to_string(Replica_gtid_state state) {
@@ -924,6 +993,14 @@ void compute_joining_replica_gtid_state(
   assert(out_missing_gtids && out_unrecoverable_gtids && out_errant_gtids &&
          out_allowed_errant_gtids);
 
+  if (!mysqlsh::dba::supports_gtid_tags(instance.get_version()) &&
+      joiner_gtids.has_tags()) {
+    throw shcore::Exception(
+        shcore::str_format("Instance '%s' doesn't support GTID tags.",
+                           instance.descr().c_str()),
+        SHERR_UNSUPPORTED_GTID_TAG);
+  }
+
   using mysqlshdk::mysql::Gtid_set;
 
   Gtid_set completely_purged_gtids;
@@ -1046,6 +1123,15 @@ std::string get_replication_source_keyword(
   }
 }
 
+const char *get_binary_logs_keyword(const mysqlshdk::utils::Version &version,
+                                    bool status) {
+  if (version < mysqlshdk::utils::Version(8, 2, 0)) {
+    return "MASTER";
+  } else {
+    return (status == true ? "BINARY LOG" : "BINARY LOGS AND GTIDS");
+  }
+}
+
 std::tuple<std::string, std::string> get_replication_source_keywords(
     const mysqlshdk::utils::Version &version) {
   if (version < mysqlshdk::utils::Version(8, 0, 23))
@@ -1054,15 +1140,66 @@ std::tuple<std::string, std::string> get_replication_source_keywords(
 }
 
 std::string get_replication_option_keyword(
-    const mysqlshdk::utils::Version &version, const std::string &option) {
-  std::string ret = option;
+    const mysqlshdk::utils::Version &version, std::string_view option) {
+  using namespace std::literals;
 
-  if (version >= mysqlshdk::utils::Version(8, 0, 26)) {
-    ret = shcore::str_replace(ret, "slave", "replica");
-    ret = shcore::str_replace(ret, "master", "source");
-  } else {
-    ret = shcore::str_replace(ret, "replica", "slave");
-    ret = shcore::str_replace(ret, "source", "master");
+  // assume current keywords (>= 8.0.26)
+  std::array<std::tuple<std::string_view, std::string_view>, 2> replacements{
+      std::make_tuple("slave"sv, "replica"sv),
+      std::make_tuple("master"sv, "source"sv)};
+
+  // switch old<>new keywords if target is an older version
+  if (version < mysqlshdk::utils::Version(8, 0, 26)) {
+    for (auto &rep : replacements)
+      std::swap(std::get<0>(rep), std::get<1>(rep));
+  }
+
+  // Cannot blindly replace keywords, otherwise "aaa_replication_bbb" is
+  // incorrectly replaced with "aaa_slavetion_bbb". Replacements are only made
+  // if:
+  //  - keyword is at the beginning and is followed with '_', '-' or ' '
+  //  - keyword is at the end and is preceded with '_', '-' or ' '
+  //  - keyword is followed and preceded with '_', '-' or ' '
+  //
+  // It's also assumed that the replacement only needs to be made once per
+  // keyword.
+
+  std::array<const char, 3> separators{'_', '-', ' '};
+
+  std::string ret{option};
+
+  for (const auto &[old_name, new_name] : replacements) {
+    if (ret == old_name) return std::string{new_name};
+
+    auto it = ret.find(old_name);
+    if (it == std::string::npos) continue;
+
+    if (it == 0) {  // replace at beginning
+      assert(ret.size() > old_name.size());
+      if (std::find(separators.begin(), separators.end(),
+                    ret[old_name.size()]) != separators.end())
+        ret.replace(it, old_name.size(), new_name);
+      continue;
+    }
+
+    if (it == (ret.size() - old_name.size())) {  // replace at end
+      assert(ret.size() > old_name.size());
+      if (std::find(separators.begin(), separators.end(), ret[it - 1]) !=
+          separators.end())
+        ret.replace(it, old_name.size(), new_name);
+      continue;
+    }
+
+    // replace in the middle
+    assert((it + old_name.size()) < ret.size());
+    if (std::find(separators.begin(), separators.end(), ret[it - 1]) ==
+        separators.end())
+      continue;
+    if (std::find(separators.begin(), separators.end(),
+                  ret[it + old_name.size()]) == separators.end())
+      continue;
+
+    ret.replace(it, old_name.size(), new_name);
   }
 
   return ret;
