@@ -134,7 +134,7 @@ Http_request::Http_request(Masked_string path, bool use_retry,
                            rest::Headers headers)
     : Request(std::move(path), std::move(headers)) {
   if (use_retry) {
-    m_retry_strategy = mysqlshdk::rest::default_retry_strategy();
+    m_retry_strategy = mysqlshdk::rest::retry_terminal_errors_strategy();
     retry_strategy = m_retry_strategy.get();
   }
 }
@@ -207,6 +207,11 @@ std::unique_ptr<IDirectory> Http_object::parent() const {
   return make_directory(m_base.real(), m_parent_config);
 }
 
+void Http_object::set_write_data(Http_request *request) {
+  request->body = m_buffer.data();
+  request->size = m_buffer.size();
+}
+
 void Http_object::close() {
   assert(is_open());
 
@@ -214,8 +219,7 @@ void Http_object::close() {
     auto request = Http_request(m_path, m_use_retry,
                                 {{"content-type", "application/octet-stream"}});
 
-    request.body = m_buffer.data();
-    request.size = m_buffer.size();
+    set_write_data(&request);
 
     throw_if_error(get_rest_service(m_base)->put(&request).get_error(),
                    "Failed to put object");
@@ -340,6 +344,11 @@ std::optional<rest::Response_error> Http_object::fetch_file_size() const {
   return {};
 }
 
+void Http_object::set_full_path(const Masked_string &full_path) {
+  m_base = get_uri_base(full_path);
+  m_path = get_uri_path(full_path);
+}
+
 void Http_directory::init_rest(const Masked_string &url) { m_url = url; }
 
 bool Http_directory::exists() const {
@@ -350,10 +359,6 @@ void Http_directory::create() {
   throw std::logic_error("Http_directory::create() - not implemented");
 }
 
-void Http_directory::close() {
-  // NO-OP
-}
-
 Masked_string Http_directory::full_path() const { return m_url; }
 
 std::unordered_set<IDirectory::File_info> Http_directory::get_file_list(
