@@ -40,7 +40,7 @@ class Enum_set {
   constexpr Enum_set() noexcept : _value(0) {}
 
   template <typename... T>
-  constexpr explicit Enum_set(Enum value, T &&... values) noexcept
+  constexpr explicit Enum_set(Enum value, T &&...values) noexcept
       : _value(ord(value)) {
     if constexpr (sizeof...(values) > 0)
       _value = ((_value | ord(values)) | ...);
@@ -85,7 +85,7 @@ class Enum_set {
   Enum_set &operator|=(Enum value) { return set(value); }
   Enum_set &operator=(Enum value) { return clear().set(value); }
 
-  constexpr operator bool() const { return !empty(); }
+  constexpr explicit operator bool() const { return !empty(); }
 
   constexpr Enum_set operator&(Enum value) const {
     return Enum_set(_value & ord(value));
