@@ -1,16 +1,18 @@
 /*
- * Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms, as
- * designated in a particular file or component or in included license
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms,
+ * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ *
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -28,6 +30,7 @@
 #include <chrono>
 #include <functional>
 #include <list>
+#include <optional>
 #include <set>
 #include <sstream>
 #include <string>
@@ -64,6 +67,9 @@ constexpr static std::size_t array_size(const T (&)[N]) noexcept {
   return N;
 }
 
+bool compare_floating_point(double valueA, double valueB,
+                            double precision) noexcept;
+
 class Scoped_callback {
  public:
   explicit Scoped_callback(std::function<void()> c) noexcept
@@ -169,21 +175,38 @@ enum class OperatingSystem {
 };
 std::string SHCORE_PUBLIC to_string(OperatingSystem os_type);
 
-bool SHCORE_PUBLIC is_valid_identifier(const std::string &name);
-mysqlshdk::db::Connection_options SHCORE_PUBLIC
-get_connection_options(const std::string &uri, bool set_defaults = true);
+bool SHCORE_PUBLIC is_valid_identifier(std::string_view name);
+
 mysqlshdk::ssh::Ssh_connection_options SHCORE_PUBLIC
 get_ssh_connection_options(const std::string &uri, bool set_defaults = true,
                            const std::string &config_path = "");
 
 std::string SHCORE_PUBLIC get_system_user();
 
-std::string SHCORE_PUBLIC strip_password(const std::string &connstring);
-
-std::string SHCORE_PUBLIC strip_ssl_args(const std::string &connstring);
+std::string SHCORE_PUBLIC strip_password(std::string_view connstring);
 
 char SHCORE_PUBLIC *mysh_get_stdin_password(const char *prompt);
 
+void SHCORE_PUBLIC split_string(std::string_view input,
+                                std::string_view separator, bool compress,
+                                auto &&cb) {
+  size_t index = 0, new_find = 0;
+  while (new_find != std::string_view::npos) {
+    new_find = input.find(separator, index);
+
+    if (new_find != std::string_view::npos) {
+      // When compress is enabled, consecutive separators
+      // do not generate new elements
+      if (new_find > index || !compress || new_find == 0)
+        if (!cb(input.substr(index, new_find - index))) return;
+
+      index = new_find + separator.length();
+    } else {
+      if (!cb(input.substr(index))) return;
+    }
+  }
+}
+
 std::vector<std::string> SHCORE_PUBLIC split_string(std::string_view input,
                                                     std::string_view separator,
                                                     bool compress = false);
@@ -195,9 +218,21 @@ bool SHCORE_PUBLIC match_glob(const std::string_view pattern,
                               const std::string_view s,
                               bool case_sensitive = false);
 
-std::string SHCORE_PUBLIC to_camel_case(const std::string &name);
-std::string SHCORE_PUBLIC from_camel_case(const std::string &name);
-std::string SHCORE_PUBLIC from_camel_case_to_dashes(const std::string &name);
+/**
+ * Replaces all escaped wildcard characters with their unescaped counterparts.
+ * If pattern already contains an unescaped wildcard characters, does not return
+ * a value.
+ *
+ * @param pattern A pattern to process.
+ *
+ * @returns Processed pattern or no value.
+ */
+std::optional<std::string> SHCORE_PUBLIC
+unescape_glob(const std::string_view pattern);
+
+std::string SHCORE_PUBLIC to_camel_case(std::string_view name);
+std::string SHCORE_PUBLIC from_camel_case(std::string_view name);
+std::string SHCORE_PUBLIC from_camel_case_to_dashes(std::string_view name);
 
 std::string SHCORE_PUBLIC errno_to_string(int err);
 
@@ -513,6 +548,10 @@ bool setenv(const std::string &name_value);
 bool unsetenv(const char *name);
 bool unsetenv(const std::string &name);
 
+void parse_schema_and_object(const std::string &str, const std::string &context,
+                             const std::string &object_type,
+                             std::string *out_schema, std::string *out_table);
+
 #ifdef _WIN32
 #define STDIN_FILENO _fileno(stdin)
 #define STDOUT_FILENO _fileno(stdout)
