@@ -24,17 +24,14 @@
  */
 
 #include "utils/utils_string.h"
-#include <algorithm>
+
 #include <array>
 #include <bitset>
 #include <cassert>
 #include <cstdarg>
-#include <cstdio>
-#include <cstdlib>
-#include <cwchar>
-#include <iomanip>
+#include <functional>
 #include <random>
-#include <sstream>
+#include <stdexcept>
 
 #ifdef _WIN32
 #include <windows.h>
@@ -146,16 +143,19 @@ std::string str_format(const char *formats, ...) {
 std::string str_replace(std::string_view s, std::string_view from,
                         std::string_view to) {
   std::string str;
-  int offs = from.length();
-  str.reserve(s.length());
 
   if (from.empty()) {
+    str.reserve(to.length() * (s.size() + 1));
+
     str.append(to);
     for (char c : s) {
       str.push_back(c);
       str.append(to);
     }
   } else {
+    str.reserve(s.length());
+
+    int offs = from.length();
     std::string::size_type start = 0, p = s.find(from);
     while (p != std::string::npos) {
       if (p > start) str.append(s, start, p - start);
@@ -198,15 +198,15 @@ size_t bits_to_string_hex_size(int nbits) {
   return num_chars;
 }
 
-std::string SHCORE_PUBLIC string_to_hex(std::string_view s) {
+std::string SHCORE_PUBLIC string_to_hex(std::string_view s, bool prefix) {
   constexpr std::array<char, 16> hexmap = {'0', '1', '2', '3', '4', '5',
                                            '6', '7', '8', '9', 'A', 'B',
                                            'C', 'D', 'E', 'F'};
 
   std::string encoded;
 
-  encoded.reserve(2 + s.size() * 2);
-  encoded.append("0x");
+  encoded.reserve((prefix ? 2 : 0) + s.size() * 2);
+  if (prefix) encoded.append("0x");
 
   for (const auto cur_char : s) {
     encoded.push_back(hexmap[(cur_char & 0xF0) >> 4]);
