@@ -29,8 +29,10 @@
 #include <map>
 #include <optional>
 #include <set>
+
 #include "mysqlshdk/include/shellcore/base_session.h"
 #include "mysqlshdk/include/shellcore/console.h"
+#include "mysqlshdk/libs/db/mysql/result.h"
 #include "mysqlshdk/libs/textui/textui.h"
 
 namespace textui = mysqlshdk::textui;
@@ -72,6 +74,7 @@ std::optional<int> find_unique_match(
   return match_index;
 }
 }  // namespace
+
 bool Command_help::execute(const std::vector<std::string> &args) {
   // Gets the entire help system
   auto help = _shell->get_helper();
@@ -200,7 +203,8 @@ std::vector<shcore::Help_topic> Command_help::get_sql_topics(
                                   name,
                                   const_cast<shcore::Help_topic *>(sql),
                                   {},
-                                  true});
+                                  true,
+                                  {}});
             entry = result->fetch_one();
           }
           // If found a specific topi we are overloading the tag field to
@@ -214,7 +218,8 @@ std::vector<shcore::Help_topic> Command_help::get_sql_topics(
                                 entry->get_string(desc_col),
                                 const_cast<shcore::Help_topic *>(sql),
                                 {},
-                                true});
+                                true,
+                                {}});
         }
       }
     }
@@ -330,9 +335,9 @@ void Command_help::print_help_global() {
                                     Topic_mask(), "detail,categories"));
 
   if (_shell->interactive_mode() == shcore::IShell_core::Mode::SQL) {
-    help->add_section("", "HELP_AVAILABLE_TOPICS_SQL", &sections, 0);
+    help->add_section({}, "", "HELP_AVAILABLE_TOPICS_SQL", &sections, 0);
   } else {
-    help->add_section("", "HELP_AVAILABLE_TOPICS_SCRIPTING", &sections, 0);
+    help->add_section({}, "", "HELP_AVAILABLE_TOPICS_SCRIPTING", &sections, 0);
   }
 
   sections.push_back(textui::bold("SHELL COMMANDS"));
@@ -357,6 +362,7 @@ void Command_help::print_help_global() {
                                  object->class_name() + "_GLOBAL",
                                  nullptr,
                                  {},
+                                 {},
                                  {}});
       }
     }
@@ -368,6 +374,7 @@ void Command_help::print_help_global() {
                              "MYSQLX_GLOBAL",
                              nullptr,
                              {},
+                             {},
                              {}});
     global_topics.push_back({"",
                              "mysql",
@@ -375,6 +382,7 @@ void Command_help::print_help_global() {
                              "MYSQL_GLOBAL",
                              nullptr,
                              {},
+                             {},
                              {}});
 
     // We need the references to use the existing formatting code
@@ -384,11 +392,14 @@ void Command_help::print_help_global() {
               shcore::Help_topic_compare());
 
     sections.push_back(textui::bold("GLOBAL OBJECTS"));
-    _shell->get_helper()->add_childs_section(global_refs, &sections, 0, false,
-                                             "GLOBALS", "GLOBALS");
-    help->add_examples_section("GLOBALS_EXAMPLE_SCRIPTING", &sections, 0);
+    _shell->get_helper()->add_childs_section({}, global_refs, &sections, 0,
+                                             false, "GLOBALS", "GLOBALS");
+    help->add_examples_section({}, "GLOBALS_EXAMPLE_SCRIPTING", &sections, 0);
   } else {
-    help->add_examples_section("GLOBALS_EXAMPLE_SQL", &sections, 0);
+    help->add_examples_section({}, "GLOBALS_EXAMPLE_SQL", &sections, 0);
+
+    help->add_section({}, "", "HELP_SQL_MODE_ADDITIONAL_FEATURES", &sections,
+                      0);
   }
 
   mysqlsh::current_console()->println(shcore::str_join(sections, "\n\n"));
