@@ -25,6 +25,8 @@
 
 #include "src/mysqlsh/prompt_handler.h"
 
+#include <type_traits>
+#include <utility>
 #include <vector>
 
 #include "mysqlshdk/include/scripting/shexcept.h"
@@ -129,8 +131,6 @@ std::string render_text_prompt(const std::string &prompt,
 std::string render_confirm_prompt(const std::string &prompt,
                                   const shcore::prompt::Prompt_options &options,
                                   std::string *out_valid_answers) {
-  std::string prompt_text;
-
   std::string clean_yes_text, clean_no_text, clean_alt_text;
   std::string yes_display_text, no_display_text, alt_display_text;
   std::string def;
@@ -180,6 +180,7 @@ std::string render_confirm_prompt(const std::string &prompt,
       def_str.append(" (default ").append(clean_alt_text).append("): ");
   }
 
+  std::string prompt_text;
   if (use_json()) {
     prompt_text = json_prompt(prompt, options);
   } else {
@@ -229,9 +230,9 @@ std::string render_select_prompt(
 }
 
 Prompt_handler::Prompt_handler(
-    const std::function<shcore::Prompt_result(bool, const char *,
-                                              std::string *)> &do_prompt_cb)
-    : m_do_prompt_cb{do_prompt_cb} {}
+    std::function<shcore::Prompt_result(bool, const char *, std::string *)>
+        do_prompt_cb) noexcept
+    : m_do_prompt_cb{std::move(do_prompt_cb)} {}
 
 shcore::Prompt_result Prompt_handler::handle_prompt(
     const std::string &prompt, const shcore::prompt::Prompt_options &options,
@@ -294,9 +295,6 @@ shcore::Prompt_result Prompt_handler::handle_select_prompt(
     std::string *response) {
   std::string prompt_text = render_select_prompt(prompt, options);
 
-  mysqlshdk::utils::nullable<std::string> good_answer;
-
-  shcore::Dictionary_t prompt_options;
   while (true) {
     auto res = m_do_prompt_cb(false, prompt_text.c_str(), response);
 
@@ -317,4 +315,4 @@ shcore::Prompt_result Prompt_handler::handle_select_prompt(
   return shcore::Prompt_result::Ok;
 }
 
-}  // namespace mysqlsh
\ No newline at end of file
+}  // namespace mysqlsh
