@@ -23,12 +23,11 @@
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "modules/adminapi/common/preconditions.h"
 #include "mysqlshdk/libs/azure/signer.h"
-#include "mysqlshdk/libs/db/replay/setup.h"
 #include "mysqlshdk/libs/utils/utils_file.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/utils_path.h"
-#include "mysqlshdk/libs/utils/utils_stacktrace.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 #include "unittest/gtest_clean.h"
 #include "unittest/mysqlshdk/libs/azure/test_utils.h"
@@ -63,7 +62,7 @@ class Auto_script_js : public Shell_js_script_tester,
     Shell_js_script_tester::set_defaults();
 
     std::string user, host, password;
-    auto connection_options = shcore::get_connection_options(_uri);
+    auto connection_options = mysqlshdk::db::Connection_options(_uri);
 
     if (connection_options.has_user()) user = connection_options.get_user();
 
@@ -223,6 +222,13 @@ class Auto_script_js : public Shell_js_script_tester,
     _options->wizards =
         strstr(GetParam().c_str(), "_interactive.") ? true : false;
 
+    // if the test concerns the AdminAPI (without the "record" suffix) and the
+    // current server version is not supported, we might as well skip it
+    if (shcore::str_beginswith(GetParam(), "js_adminapi") &&
+        !shcore::str_endswith(GetParam(), "_norecord")) {
+      if (!Shell_test_env::check_min_version_skip_test()) return;
+    }
+
     SCOPED_TRACE(GetParam());
 
     std::string folder;
@@ -239,14 +245,6 @@ class Auto_script_js : public Shell_js_script_tester,
       execute_setup();
     }
 
-    // todo(kg): _norecord files haven't defined functions from script.js, e.g.
-    // `EXPECT_NE`. reset_replayable_shell call reset_shell therefore we lost
-    // those previously defined and called in constructor functions.
-    // I run setup.js script here once again, but this should be done somewhere
-    // else, but I don't know where.
-    set_setup_script(shcore::path::join_path(g_test_home, "scripts", "setup_js",
-                                             "setup.js"));
-
     if (g_mysqld_path_variables && folder == "js_mixed_versions") {
       auto variables = shcore::str_split(g_mysqld_path_variables, ",");
 
@@ -278,9 +276,6 @@ class Auto_script_js : public Shell_js_script_tester,
       }
     }
 
-    const std::vector<std::string> argv;
-    _interactive_shell->process_file(_setup_script, argv);
-
     execute("const __script_file = '" + GetParam() + "'");
 
     set_config_folder("auto/" + folder);
@@ -444,7 +439,7 @@ class Credential_store_test : public Auto_script_js {
  private:
   void prepare_session() {
     m_session = mysqlshdk::db::mysql::Session::create();
-    m_session->connect(shcore::get_connection_options(_mysql_uri));
+    m_session->connect(mysqlshdk::db::Connection_options(_mysql_uri));
     const auto &connection_options = m_session->get_connection_options();
     if (connection_options.has_host()) m_host = connection_options.get_host();
     m_session->executef("CREATE USER ?@? IDENTIFIED BY ?", k_first_user, m_host,
