@@ -62,7 +62,7 @@ class Auto_script_py : public Shell_py_script_tester,
     Shell_py_script_tester::set_defaults();
 
     std::string user, host, password;
-    auto connection_options = shcore::get_connection_options(_uri);
+    auto connection_options = mysqlshdk::db::Connection_options(_uri);
 
     if (connection_options.has_user()) user = connection_options.get_user();
 
@@ -197,6 +197,13 @@ TEST_P(Auto_script_py, run_and_check) {
   _options->wizards =
       strstr(GetParam().c_str(), "_interactive.") ? true : false;
 
+  // if the test concerns the AdminAPI (without the "record" suffix) and the
+  // current server version is not supported, we might as well skip it
+  if (shcore::str_beginswith(GetParam(), "py_adminapi") &&
+      !shcore::str_endswith(GetParam(), "_norecord")) {
+    if (!Shell_test_env::check_min_version_skip_test()) return;
+  }
+
   SCOPED_TRACE(GetParam());
 
   std::string folder;
