@@ -1,16 +1,18 @@
 /*
- * Copyright (c) 2017, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms, as
- * designated in a particular file or component or in included license
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms,
+ * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ *
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -538,7 +540,7 @@ TEST_F(Completer_frontend, builtin_others) {
                       "\\show",    "\\source",     "\\sql",        "\\status",
                       "\\system",  "\\use",        "\\warnings",   "\\watch"});
 
-#ifndef HAVE_V8
+#ifndef HAVE_JS
   expect.erase(std::find(expect.begin(), expect.end(), "\\js"));
 #endif
 
@@ -621,7 +623,7 @@ TEST_F(Completer_frontend, sql_table_o3) {
   EXPECT_AFTER_TAB("describe `pl", "describe `plugin`");
 }
 
-#ifdef HAVE_V8
+#ifdef HAVE_JS
 TEST_F(Completer_frontend, js_keywords) {
   execute("\\js");
 
@@ -741,26 +743,16 @@ TEST_F(Completer_frontend, js_adminapi) {
   CHECK_OBJECT_COMPLETIONS("dba");
 
   // TS_FR5.2_C02, TS_FR5.2_X02
-  EXPECT_AFTER_TAB_TAB("dba.", strv({"checkInstanceConfiguration()",
-                                     "configureInstance()",
-                                     "configureLocalInstance()",
-                                     "configureReplicaSetInstance()",
-                                     "createCluster()",
-                                     "createReplicaSet()",
-                                     "deleteSandboxInstance()",
-                                     "deploySandboxInstance()",
-                                     "dropMetadataSchema()",
-                                     "getCluster()",
-                                     "getClusterSet()",
-                                     "getReplicaSet()",
-                                     "help()",
-                                     "killSandboxInstance()",
-                                     "rebootClusterFromCompleteOutage()",
-                                     "session",
-                                     "startSandboxInstance()",
-                                     "stopSandboxInstance()",
-                                     "upgradeMetadata()",
-                                     "verbose"}));
+  EXPECT_AFTER_TAB_TAB(
+      "dba.",
+      strv({"checkInstanceConfiguration()", "configureInstance()",
+            "configureReplicaSetInstance()", "createCluster()",
+            "createReplicaSet()", "deleteSandboxInstance()",
+            "deploySandboxInstance()", "dropMetadataSchema()", "getCluster()",
+            "getClusterSet()", "getReplicaSet()", "help()",
+            "killSandboxInstance()", "rebootClusterFromCompleteOutage()",
+            "session", "startSandboxInstance()", "stopSandboxInstance()",
+            "upgradeMetadata()", "verbose"}));
   EXPECT_AFTER_TAB("dba.depl", "dba.deploySandboxInstance()");
 }
 
@@ -908,7 +900,6 @@ TEST_F(Completer_frontend, js_devapi_collection) {
   CHECK_OBJECT_COMPLETIONS("people.find().groupBy(['x'])");
   CHECK_OBJECT_COMPLETIONS("people.find().groupBy(['x']).having('x')");
   CHECK_OBJECT_COMPLETIONS("people.find().limit(2)");
-  CHECK_OBJECT_COMPLETIONS("people.find().limit(0).skip(2)");
   CHECK_OBJECT_COMPLETIONS("people.find().lockShared()");
   CHECK_OBJECT_COMPLETIONS("people.find().lockExclusive()");
   CHECK_OBJECT_COMPLETIONS("people.find(':x').bind('x',1)");
@@ -1154,7 +1145,7 @@ TEST_F(Completer_frontend, js_devapi_members_classic) {
       {"help", ""},
       {"close", ""},
       {"runSql", "('select 1')"},
-      {"query", "('select 1')"},
+      {"setQueryAttributes", "({})"},
       {"_getSocketFd", ""}};
   CHECK_OBJECT_MEMBER_COMPLETIONS("session", session_calls);
 }
@@ -1280,26 +1271,16 @@ TEST_F(Completer_frontend, py_adminapi) {
   CHECK_OBJECT_COMPLETIONS("dba");
 
   // TS_FR5.2_C02, TS_FR5.2_X02
-  EXPECT_AFTER_TAB_TAB("dba.", strv({"check_instance_configuration()",
-                                     "configure_instance()",
-                                     "configure_local_instance()",
-                                     "configure_replica_set_instance()",
-                                     "create_cluster()",
-                                     "create_replica_set()",
-                                     "delete_sandbox_instance()",
-                                     "deploy_sandbox_instance()",
-                                     "drop_metadata_schema()",
-                                     "get_cluster()",
-                                     "get_cluster_set()",
-                                     "get_replica_set()",
-                                     "help()",
-                                     "kill_sandbox_instance()",
-                                     "reboot_cluster_from_complete_outage()",
-                                     "session",
-                                     "start_sandbox_instance()",
-                                     "stop_sandbox_instance()",
-                                     "upgrade_metadata()",
-                                     "verbose"}));
+  EXPECT_AFTER_TAB_TAB(
+      "dba.",
+      strv({"check_instance_configuration()", "configure_instance()",
+            "configure_replica_set_instance()", "create_cluster()",
+            "create_replica_set()", "delete_sandbox_instance()",
+            "deploy_sandbox_instance()", "drop_metadata_schema()",
+            "get_cluster()", "get_cluster_set()", "get_replica_set()", "help()",
+            "kill_sandbox_instance()", "reboot_cluster_from_complete_outage()",
+            "session", "start_sandbox_instance()", "stop_sandbox_instance()",
+            "upgrade_metadata()", "verbose"}));
   EXPECT_AFTER_TAB("dba.depl", "dba.deploy_sandbox_instance()");
 }
 
@@ -1444,7 +1425,6 @@ TEST_F(Completer_frontend, py_devapi_collection) {
   CHECK_OBJECT_COMPLETIONS("people.find().group_by(['x'])");
   CHECK_OBJECT_COMPLETIONS("people.find().group_by(['x']).having('x')");
   CHECK_OBJECT_COMPLETIONS("people.find().limit(2)");
-  CHECK_OBJECT_COMPLETIONS("people.find().limit(0).skip(2)");
   CHECK_OBJECT_COMPLETIONS("people.find(':x').bind('x',1)");
   CHECK_OBJECT_COMPLETIONS("people.find().execute()");
 
