@@ -40,9 +40,11 @@ class Mock_mysql_shell : public mysqlsh::Mysql_shell {
                    shcore::Interpreter_delegate *custom_delegate)
       : mysqlsh::Mysql_shell(options, custom_delegate) {}
 
-  MOCK_METHOD4(connect, std::shared_ptr<mysqlsh::ShellBaseSession>(
-                            const mysqlshdk::db::Connection_options &, bool,
-                            bool, bool));
+  MOCK_METHOD4(
+      connect,
+      std::shared_ptr<mysqlsh::ShellBaseSession>(
+          const mysqlshdk::db::Connection_options &, bool,
+          std::function<void(std::shared_ptr<mysqlshdk::db::ISession>)>, bool));
   MOCK_METHOD1(cmd_disconnect, bool(const std::vector<std::string> &args));
 };
 
@@ -66,6 +68,7 @@ class mod_shell_test : public Shell_core_test_wrapper {
   std::shared_ptr<mysqlsh::Shell> _shell;
 };
 
+// clang-format off
 TEST_F(mod_shell_test, parse_uri) {
   {
     std::string args;
@@ -283,16 +286,68 @@ TEST_F(mod_shell_test, parse_uri) {
         "ECDSA-AES256-GCM-SHA384&tls-version=TLSv1.1%2CTLSv1.2",
         _shell->unparse_uri(dict));
   }
+  // clang-format on
+
+#ifdef _WIN32
+  {
+    std::string args;
+    args = "user@host?plugin-authentication-kerberos-client-mode=SSPI";
+
+    auto dict = _shell->parse_uri(args);
+
+    EXPECT_TRUE(dict->has_key(mysqlshdk::db::kKerberosClientAuthMode));
+    EXPECT_STREQ(
+        mysqlshdk::db::kKerberosAuthModeSSPI,
+        dict->get_string(mysqlshdk::db::kKerberosClientAuthMode).c_str());
+
+    // test for case-sensitivity
+    args = "user@host?plugin-authentication-kerberos-client-mode=GsSapi";
+
+    dict = _shell->parse_uri(args);
+
+    EXPECT_TRUE(dict->has_key(mysqlshdk::db::kKerberosClientAuthMode));
+    EXPECT_STREQ(
+        mysqlshdk::db::kKerberosAuthModeGSSAPI,
+        dict->get_string(mysqlshdk::db::kKerberosClientAuthMode).c_str());
+  }
+#else
+  {
+    std::string args;
+    args = "user@host?plugin-authentication-kerberos-client-mode=SSPI";
+
+    EXPECT_THROW_MSG_CONTAINS(_shell->parse_uri(args), std::exception,
+                              "Invalid URI: Invalid connection option "
+                              "'plugin-authentication-kerberos-client-mode'.");
+  }
+#endif
+
+  {
+    std::string args;
+    args =
+        "user@host?oci-config-file=config.cfg&authentication-oci-client-config-"
+        "profile=name";
+
+    auto dict = _shell->parse_uri(args);
+
+    EXPECT_TRUE(dict->has_key(mysqlshdk::db::kOciConfigFile));
+    EXPECT_TRUE(
+        dict->has_key(mysqlshdk::db::kOciAuthenticationClientConfigProfile));
+
+    EXPECT_STREQ("config.cfg",
+                 dict->get_string(mysqlshdk::db::kOciConfigFile).c_str());
+    EXPECT_STREQ(
+        "name",
+        dict->get_string(mysqlshdk::db::kOciAuthenticationClientConfigProfile)
+            .c_str());
+  }
 }
 
 TEST_F(mod_shell_test, connect) {
-  // ensure that shell.connect() calls Mysql_shell::connect()
-
-  EXPECT_CALL(*_backend, connect(_, false, true, true));
+  EXPECT_CALL(*_backend, connect(_, true, _, _));
 
   _shell->connect(mysqlshdk::db::Connection_options{_mysql_uri});
 
-  EXPECT_CALL(*_backend, connect(_, false, false, true));
+  EXPECT_CALL(*_backend, connect(_, false, _, _));
   _shell->open_session(mysqlshdk::db::Connection_options{_mysql_uri});
 }
 
@@ -442,5 +497,4 @@ TEST_F(mod_shell_test, dump_rows) {
     }
   }
 }
-
 }  // namespace testing
