@@ -39,6 +39,7 @@ session.close();
 //@<> WL13236-TSFR1_2 {VER(<8.0.23)}
 function expect_rs_variable() {
   EXPECT_STDOUT_CONTAINS_MULTILINE(`You are connected to a member of replicaset 'rs'.
+Switching to JavaScript mode...
 {
     "replicaSet": {
         "name": "rs",
@@ -61,8 +62,9 @@ function expect_rs_variable() {
                     "applierThreadState": "Slave has read all relay log; waiting for more updates",
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for master to send event",
-                    "replicationLag": null,
-                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
+                    "replicationLag": "applier_queue_applied",
+                    "replicationSsl": "TLS_[[*]]",
+                    "replicationSslMode": "REQUIRED"
                 },
                 "status": "ONLINE"
             }
@@ -75,6 +77,7 @@ function expect_rs_variable() {
 //@<> WL13236-TSFR1_2 parallel-appliers {VER(>=8.0.23)}
 function expect_rs_variable() {
   EXPECT_STDOUT_CONTAINS_MULTILINE(`You are connected to a member of replicaset 'rs'.
+Switching to JavaScript mode...
 {
     "replicaSet": {
         "name": "rs",
@@ -98,8 +101,9 @@ function expect_rs_variable() {
                     "applierWorkerThreads": 4,
                     "receiverStatus": "ON",
                     "receiverThreadState": "Waiting for ${__source_keyword} to send event",
-                    "replicationLag": null,
-                    "replicationSsl": "TLS_AES_256_GCM_SHA384 TLSv1.3"
+                    "replicationLag": "applier_queue_applied",
+                    "replicationSsl": "TLS_[[*]]",
+                    "replicationSslMode": "REQUIRED"
                 },
                 "status": "ONLINE"
             }
@@ -150,13 +154,13 @@ ERROR: RuntimeError: An open session is required to perform this operation.`);
 // WL13236-TSFR2_2: Start the Shell with a valid connection data of an instance that is secondary member of a ReplicaSet and use the --redirect-primary option. Validate that the the active session is created on the primary.
 
 //@<> WL13236-TSFR2_2
-mysqlsh([__hostname_ip_uri2, '--redirect-primary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_uri2, '--redirect-primary', '--js', '--execute', 'println(session)']);
 
 EXPECT_STDOUT_CONTAINS_MULTILINE(`Reconnecting to the PRIMARY instance of an InnoDB ReplicaSet (${hostname_ip}:${__mysql_sandbox_port1})...
 <ClassicSession:root@${hostname_ip}:${__mysql_sandbox_port1}>`);
 
 //@<> WL13236-TSFR2_2 - X
-mysqlsh([__hostname_ip_x_uri2, '--redirect-primary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_x_uri2, '--redirect-primary', '--js', '--execute', 'println(session)']);
 
 EXPECT_STDOUT_CONTAINS_MULTILINE(`Reconnecting to the PRIMARY instance of an InnoDB ReplicaSet (${hostname_ip}:${__mysql_sandbox_x_port1})...
 <Session:root@${hostname_ip}:${__mysql_sandbox_x_port1}>`);
@@ -179,14 +183,14 @@ function expect_already_primary() {
   EXPECT_STDOUT_CONTAINS('NOTE: --redirect-primary ignored because target is already a PRIMARY');
 }
 
-mysqlsh([__hostname_ip_uri1, '--redirect-primary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_uri1, '--redirect-primary', '--js', '--execute', 'println(session)']);
 
 expect_already_primary();
 EXPECT_STDOUT_CONTAINS(`<ClassicSession:root@${hostname_ip}:${__mysql_sandbox_port1}>`);
 
 ////////////////////////////////////////////////////////////////////////////////
 //@<> connect to primary, use --redirect-primary, expect no error - X
-mysqlsh([__hostname_ip_x_uri1, '--redirect-primary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_x_uri1, '--redirect-primary', '--js', '--execute', 'println(session)']);
 
 expect_already_primary();
 EXPECT_STDOUT_CONTAINS(`<Session:root@${hostname_ip}:${__mysql_sandbox_x_port1}>`);
@@ -209,7 +213,7 @@ expect_rs_variable();
 //@<> WL13236-TSFR2_3
 function expect_error_redirect_primary_missing_metadata() {
   EXPECT_STDOUT_CONTAINS('While handling --redirect-primary:');
-  EXPECT_STDOUT_CONTAINS('Metadata schema of an InnoDB cluster or ReplicaSet not found');
+  EXPECT_STDOUT_CONTAINS('Metadata schema of an InnoDB Cluster or ReplicaSet not found');
 }
 
 mysqlsh([__hostname_ip_uri3, '--redirect-primary', '--execute', 'println(session)']);
@@ -238,7 +242,7 @@ expect_error_redirect_primary_missing_metadata();
 
 //@<> WL13236-TSFR2_4
 function expect_error_redirect_primary_no_session() {
-  EXPECT_STDOUT_CONTAINS('ERROR: The --redirect-primary option requires a session to a member of an InnoDB cluster or ReplicaSet.');
+  EXPECT_STDOUT_CONTAINS('ERROR: The --redirect-primary option requires a session to a member of an InnoDB Cluster or ReplicaSet.');
 }
 
 mysqlsh(['--redirect-primary', '--execute', 'println(session)']);
@@ -255,13 +259,13 @@ expect_error_redirect_primary_no_session();
 // WL13236-TSFR3_2: Start the Shell with a valid connection data of an instance that is the primary member of a ReplicaSet and use the --redirect-secondary option. Validate that the the active session is created on a secondary instance.
 
 //@<> WL13236-TSFR3_2
-mysqlsh([__hostname_ip_uri1, '--redirect-secondary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_uri1, '--redirect-secondary', '--js', '--execute', 'println(session)']);
 
 EXPECT_STDOUT_CONTAINS_MULTILINE(`Reconnecting to the SECONDARY instance of an InnoDB ReplicaSet (${hostname_ip}:${__mysql_sandbox_port2})...
 <ClassicSession:root@${hostname_ip}:${__mysql_sandbox_port2}>`);
 
 //@<> WL13236-TSFR3_2 - X
-mysqlsh([__hostname_ip_x_uri1, '--redirect-secondary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_x_uri1, '--redirect-secondary', '--js', '--execute', 'println(session)']);
 
 EXPECT_STDOUT_CONTAINS_MULTILINE(`Reconnecting to the SECONDARY instance of an InnoDB ReplicaSet (${hostname_ip}:${__mysql_sandbox_x_port2})...
 <Session:root@${hostname_ip}:${__mysql_sandbox_x_port2}>`);
@@ -284,14 +288,14 @@ function expect_already_secondary() {
   EXPECT_STDOUT_CONTAINS('NOTE: --redirect-secondary ignored because target is already a SECONDARY');
 }
 
-mysqlsh([__hostname_ip_uri2, '--redirect-secondary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_uri2, '--redirect-secondary', '--js', '--execute', 'println(session)']);
 
 expect_already_secondary();
 EXPECT_STDOUT_CONTAINS(`<ClassicSession:root@${hostname_ip}:${__mysql_sandbox_port2}>`);
 
 ////////////////////////////////////////////////////////////////////////////////
 //@<> connect to secondary, use --redirect-secondary, expect no error - X
-mysqlsh([__hostname_ip_x_uri2, '--redirect-secondary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_x_uri2, '--redirect-secondary', '--js', '--execute', 'println(session)']);
 
 expect_already_secondary();
 EXPECT_STDOUT_CONTAINS(`<Session:root@${hostname_ip}:${__mysql_sandbox_x_port2}>`);
@@ -314,7 +318,7 @@ expect_rs_variable();
 //@<> WL13236-TSFR3_3
 function expect_error_redirect_secondary_missing_metadata() {
   EXPECT_STDOUT_CONTAINS('While handling --redirect-secondary:');
-  EXPECT_STDOUT_CONTAINS('Metadata schema of an InnoDB cluster or ReplicaSet not found');
+  EXPECT_STDOUT_CONTAINS('Metadata schema of an InnoDB Cluster or ReplicaSet not found');
 }
 
 mysqlsh([__hostname_ip_uri3, '--redirect-secondary', '--execute', 'println(session)']);
@@ -343,7 +347,7 @@ expect_error_redirect_secondary_missing_metadata();
 
 //@<> WL13236-TSFR3_4
 function expect_error_redirect_secondary_no_session() {
-  EXPECT_STDOUT_CONTAINS('ERROR: The --redirect-secondary option requires a session to a member of an InnoDB cluster or ReplicaSet.');
+  EXPECT_STDOUT_CONTAINS('ERROR: The --redirect-secondary option requires a session to a member of an InnoDB Cluster or ReplicaSet.');
 }
 
 mysqlsh(['--redirect-secondary', '--execute', 'println(session)']);
@@ -359,7 +363,7 @@ expect_error_redirect_secondary_no_session();
 ////////////////////////////////////////////////////////////////////////////////
 //@<> use --cluster with a replicaset member, expect error
 function expect_error_cluster_with_replicaset() {
-  EXPECT_STDOUT_CONTAINS('Option --cluster requires a session to a member of an InnoDB cluster.');
+  EXPECT_STDOUT_CONTAINS('Option --cluster requires a session to a member of an InnoDB Cluster.');
   EXPECT_STDOUT_CONTAINS('This function is not available through a session to an instance that is a member of an InnoDB ReplicaSet');
 }
 
@@ -441,12 +445,12 @@ function expect_error_no_secondaries() {
   EXPECT_STDOUT_CONTAINS('No secondaries found in an InnoDB ReplicaSet');
 }
 
-mysqlsh([__hostname_ip_uri1, '--redirect-secondary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_uri1, '--redirect-secondary', '--js', '--execute', 'println(session)']);
 
 expect_error_no_secondaries();
 
 //@<> connect to primary, try to redirect to secondary, expect error - X
-mysqlsh([__hostname_ip_x_uri1, '--redirect-secondary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_x_uri1, '--redirect-secondary', '--js', '--execute', 'println(session)']);
 
 expect_error_no_secondaries();
 
@@ -466,12 +470,12 @@ function expect_error_no_primaries() {
   EXPECT_STDOUT_CONTAINS('No primaries found in an InnoDB ReplicaSet');
 }
 
-mysqlsh([__hostname_ip_uri2, '--redirect-primary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_uri2, '--redirect-primary', '--js', '--execute', 'println(session)']);
 
 expect_error_no_primaries();
 
 //@<> connect to secondary, try to redirect to primary, expect error - X
-mysqlsh([__hostname_ip_x_uri2, '--redirect-primary', '--execute', 'println(session)']);
+mysqlsh([__hostname_ip_x_uri2, '--redirect-primary', '--js', '--execute', 'println(session)']);
 
 expect_error_no_primaries();
 
