@@ -969,9 +969,9 @@ Create Table: CREATE TABLE `my_coll` (
 ||Duplicate key name 'myIndex' (MySQL Error 1061)
 
 //@ Create an index with a not valid JSON document definition. (WL10858-FR5_3)
-||SyntaxError: Invalid shorthand property initializer
-||SyntaxError: Unexpected token ']'
-||SyntaxError: Unexpected token '}'
+||Invalid shorthand property name initializer in object literal (SyntaxError)
+||Expected comma but found ] (SyntaxError)
+||Expected comma but found } (SyntaxError)
 
 //@ Create an index where its definition is a JSON document but its structure is not valid. (WL10858-FR5_4)
 ||Invalid number of arguments, expected value for 'fields[0].field' (MySQL Error 5015)
