@@ -182,12 +182,6 @@ PROPERTIES
       executionTime
             Same as getExecutionTime
 
-      warningCount
-            Same as getWarningCount
-
-            ATTENTION: This property will be removed in a future release, use
-                       the warningsCount property instead.
-
       warnings
             Same as getWarnings
 
@@ -202,12 +196,6 @@ FUNCTIONS
             Retrieves a string value indicating the execution time of the
             executed operation.
 
-      getWarningCount()
-            The number of warnings produced by the last statement execution.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the getWarningsCount function instead.
-
       getWarnings()
             Retrieves the warnings generated by the executed operation.
 
@@ -362,13 +350,6 @@ FUNCTIONS
             Sets number of documents to skip on the resultset when a limit has
             been defined.
 
-      skip(numberOfDocs)
-            Sets number of documents to skip on the resultset when a limit has
-            been defined.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the offset() function instead.
-
       sort(...)
             Sets the sorting criteria to be used on the DocResult.
 
@@ -386,13 +367,6 @@ FUNCTIONS
             Appends a value into an array attribute in documents of a
             collection.
 
-      arrayDelete(docPath)
-            Deletes the value at a specific position in an array attribute in
-            documents of a collection.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the unset() function instead.
-
       arrayInsert(docPath, value)
             Inserts a value into a specific position in an array attribute in
             documents of a collection.
@@ -412,13 +386,6 @@ FUNCTIONS
             Sets a limit for the documents to be updated by the operations
             added to the handler.
 
-      merge(document)
-            Adds attributes taken from a document into the documents in a
-            collection.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the patch() function instead.
-
       modify(searchCondition)
             Sets the search condition to identify the Documents to be updated
             on the owner Collection.
@@ -516,12 +483,6 @@ PROPERTIES
       executionTime
             Same as getExecutionTime
 
-      warningCount
-            Same as getWarningCount
-
-            ATTENTION: This property will be removed in a future release, use
-                       the warningsCount property instead.
-
       warnings
             Same as getWarnings
 
@@ -543,12 +504,6 @@ FUNCTIONS
             Retrieves a string value indicating the execution time of the
             executed operation.
 
-      getWarningCount()
-            The number of warnings produced by the last statement execution.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the getWarningsCount function instead.
-
       getWarnings()
             Retrieves the warnings generated by the executed operation.
 
@@ -576,12 +531,6 @@ DESCRIPTION
       - Transaction handling functions
 
 PROPERTIES
-      affectedItemCount
-            Same as getAffectedItemCount
-
-            ATTENTION: This property will be removed in a future release, use
-                       the affectedItemsCount property instead.
-
       affectedItemsCount
             Same as getAffectedItemsCount
 
@@ -594,12 +543,6 @@ PROPERTIES
       generatedIds
             Same as getGeneratedIds.
 
-      warningCount
-            Same as getWarningCount
-
-            ATTENTION: This property will be removed in a future release, use
-                       the warningsCount property instead.
-
       warnings
             Same as getWarnings
 
@@ -607,12 +550,6 @@ PROPERTIES
             Same as getWarningsCount
 
 FUNCTIONS
-      getAffectedItemCount()
-            The the number of affected items for the last operation.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the getAffectedItemsCount function instead.
-
       getAffectedItemsCount()
             The the number of affected items for the last operation.
 
@@ -626,12 +563,6 @@ FUNCTIONS
       getGeneratedIds()
             Returns the list of document ids generated on the server.
 
-      getWarningCount()
-            The number of warnings produced by the last statement execution.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the getWarningsCount function instead.
-
       getWarnings()
             Retrieves the warnings generated by the executed operation.
 
@@ -665,12 +596,6 @@ PROPERTIES
       executionTime
             Same as getExecutionTime
 
-      warningCount
-            Same as getWarningCount
-
-            ATTENTION: This property will be removed in a future release, use
-                       the warningsCount property instead.
-
       warnings
             Same as getWarnings
 
@@ -704,12 +629,6 @@ FUNCTIONS
             Retrieves a string value indicating the execution time of the
             executed operation.
 
-      getWarningCount()
-            The number of warnings produced by the last statement execution.
-
-            ATTENTION: This function will be removed in a future release, use
-                       the getWarningsCount function instead.
-
       getWarnings()
             Retrieves the warnings generated by the executed operation.
 
