@@ -379,6 +379,14 @@ DESCRIPTION
         at the PERFORMANCE_SCHEMA connection attributes tables.
       - local-infile: Enable/disable LOAD DATA LOCAL INFILE.
       - net-buffer-length: The buffer size for TCP/IP and socket communication.
+      - plugin-authentication-kerberos-client-mode: (Windows) Allows defining
+        the kerberos client mode (SSPI, GSSAPI) when using kerberos
+        authentication.
+      - oci-config-file: Allows defining the OCI configuration file for OCI
+        authentication.
+      - authentication-oci-client-config-profile: Allows defining the OCI
+        profile used from the configuration for client side OCI authentication.
+      - client-interactive: Enable/disable interactive mode for the session.
 
       When these options are defined in a URI, their values must be URL
       encoded.
@@ -390,9 +398,7 @@ DESCRIPTION
       - uri: a URI string.
       - scheme: the protocol to be used on the connection.
       - user: the MySQL user name to be used on the connection.
-      - dbUser: alias for user.
       - password: the password to be used on the connection.
-      - dbPassword: same as password.
       - host: the hostname or IP address to be used on the connection.
       - port: the port to be used in a TCP connection.
       - socket: the socket file name to be used on a connection through unix
@@ -408,9 +414,6 @@ DESCRIPTION
       - ssh-config-file: the SSH configuration file, default is the value of
         shell.options['ssh.configFile']
 
-      ATTENTION: The dbUser and dbPassword options are will be removed in a
-                 future release.
-
       ATTENTION: The connection options have precedence over options specified
                  in the connection options uri
 
@@ -497,6 +500,14 @@ DESCRIPTION
         at the PERFORMANCE_SCHEMA connection attributes tables.
       - local-infile: Enable/disable LOAD DATA LOCAL INFILE.
       - net-buffer-length: The buffer size for TCP/IP and socket communication.
+      - plugin-authentication-kerberos-client-mode: (Windows) Allows defining
+        the kerberos client mode (SSPI, GSSAPI) when using kerberos
+        authentication.
+      - oci-config-file: Allows defining the OCI configuration file for OCI
+        authentication.
+      - authentication-oci-client-config-profile: Allows defining the OCI
+        profile used from the configuration for client side OCI authentication.
+      - client-interactive: Enable/disable interactive mode for the session.
 
       When these options are defined in a URI, their values must be URL
       encoded.
@@ -508,9 +519,7 @@ DESCRIPTION
       - uri: a URI string.
       - scheme: the protocol to be used on the connection.
       - user: the MySQL user name to be used on the connection.
-      - dbUser: alias for user.
       - password: the password to be used on the connection.
-      - dbPassword: same as password.
       - host: the hostname or IP address to be used on the connection.
       - port: the port to be used in a TCP connection.
       - socket: the socket file name to be used on a connection through unix
@@ -526,9 +535,6 @@ DESCRIPTION
       - ssh-config-file: the SSH configuration file, default is the value of
         shell.options['ssh.configFile']
 
-      ATTENTION: The dbUser and dbPassword options are will be removed in a
-                 future release.
-
       ATTENTION: The connection options have precedence over options specified
                  in the connection options uri
 
@@ -1384,6 +1390,13 @@ The following options are valid for use either in a URI or in a dictionary:
   PERFORMANCE_SCHEMA connection attributes tables.
 - local-infile: Enable/disable LOAD DATA LOCAL INFILE.
 - net-buffer-length: The buffer size for TCP/IP and socket communication.
+- plugin-authentication-kerberos-client-mode: (Windows) Allows defining the
+  kerberos client mode (SSPI, GSSAPI) when using kerberos authentication.
+- oci-config-file: Allows defining the OCI configuration file for OCI
+  authentication.
+- authentication-oci-client-config-profile: Allows defining the OCI profile
+  used from the configuration for client side OCI authentication.
+- client-interactive: Enable/disable interactive mode for the session.
 
 When these options are defined in a URI, their values must be URL encoded.
 
@@ -1394,9 +1407,7 @@ Base Connection Options
 - uri: a URI string.
 - scheme: the protocol to be used on the connection.
 - user: the MySQL user name to be used on the connection.
-- dbUser: alias for user.
 - password: the password to be used on the connection.
-- dbPassword: same as password.
 - host: the hostname or IP address to be used on the connection.
 - port: the port to be used in a TCP connection.
 - socket: the socket file name to be used on a connection through unix sockets.
@@ -1411,9 +1422,6 @@ SSH Tunnel Connection Options
 - ssh-config-file: the SSH configuration file, default is the value of
   shell.options['ssh.configFile']
 
-ATTENTION: The dbUser and dbPassword options are will be removed in a future
-           release.
-
 ATTENTION: The connection options have precedence over options specified in the
            connection options uri
 
@@ -1657,6 +1665,13 @@ The following options are valid for use either in a URI or in a dictionary:
   PERFORMANCE_SCHEMA connection attributes tables.
 - local-infile: Enable/disable LOAD DATA LOCAL INFILE.
 - net-buffer-length: The buffer size for TCP/IP and socket communication.
+- plugin-authentication-kerberos-client-mode: (Windows) Allows defining the
+  kerberos client mode (SSPI, GSSAPI) when using kerberos authentication.
+- oci-config-file: Allows defining the OCI configuration file for OCI
+  authentication.
+- authentication-oci-client-config-profile: Allows defining the OCI profile
+  used from the configuration for client side OCI authentication.
+- client-interactive: Enable/disable interactive mode for the session.
 
 When these options are defined in a URI, their values must be URL encoded.
 
@@ -1667,9 +1682,7 @@ Base Connection Options
 - uri: a URI string.
 - scheme: the protocol to be used on the connection.
 - user: the MySQL user name to be used on the connection.
-- dbUser: alias for user.
 - password: the password to be used on the connection.
-- dbPassword: same as password.
 - host: the hostname or IP address to be used on the connection.
 - port: the port to be used in a TCP connection.
 - socket: the socket file name to be used on a connection through unix sockets.
@@ -1684,9 +1697,6 @@ SSH Tunnel Connection Options
 - ssh-config-file: the SSH configuration file, default is the value of
   shell.options['ssh.configFile']
 
-ATTENTION: The dbUser and dbPassword options are will be removed in a future
-           release.
-
 ATTENTION: The connection options have precedence over options specified in the
            connection options uri
 
