@@ -1,4 +1,6 @@
 var sandbox_uris = {}
+// script should be found, atm it executes 3 times when running a single JS test
+var PLUGIN_ALREADY_EXISTS = 1125;
 sandbox_uris[__mysql_sandbox_port1] = __sandbox_uri1;
 sandbox_uris[__mysql_sandbox_port2] = __sandbox_uri2;
 sandbox_uris[__mysql_sandbox_port3] = __sandbox_uri3;
@@ -222,7 +224,7 @@ function hasAwsEnvironment() {
 
 
 function hasAuthEnvironment(context) {
-  if (['LDAP_SIMPLE', 'LDAP_SASL', 'LDAP_KERBEROS', 'KERBEROS', 'FIDO', 'OCI_AUTH'].indexOf(context) == -1) {
+  if (['LDAP_SIMPLE', 'LDAP_SASL', 'LDAP_KERBEROS', 'KERBEROS', 'FIDO', 'WEBAUTHN', 'OCI_AUTH'].indexOf(context) == -1) {
     return false
   }
 
@@ -282,18 +284,14 @@ function debugAuthPlugins() {
 
 
 function getAuthServerConfig(context) {
-  if (!isAuthMethodSupported(context)) {
-    return {};
-  }
-
   const ext = __os_type == "windows" ? "dll" : "so";
 
   if (context == 'LDAP_SIMPLE') {
     var server_conf = {
       "plugin-load-add": `authentication_ldap_simple.${ext}`,
-      "authentication_ldap_simple_server_host": `${LDAP_SIMPLE_SERVER_HOST}`,
-      "authentication_ldap_simple_server_port": parseInt(`${LDAP_SIMPLE_SERVER_PORT}`),
-      "authentication_ldap_simple_bind_base_dn": `${LDAP_SIMPLE_BIND_BASE_DN}`
+      "loose-authentication_ldap_simple_server_host": `${LDAP_SIMPLE_SERVER_HOST}`,
+      "loose-authentication_ldap_simple_server_port": parseInt(`${LDAP_SIMPLE_SERVER_PORT}`),
+      "loose-authentication_ldap_simple_bind_base_dn": `${LDAP_SIMPLE_BIND_BASE_DN}`
     };
 
     if (__os_type == 'windows') {
@@ -305,25 +303,25 @@ function getAuthServerConfig(context) {
   } else if (context == 'LDAP_SASL') {
     return {
       "plugin-load-add": `authentication_ldap_sasl.${ext}`,
-      "authentication_ldap_sasl_server_host": `${LDAP_SASL_SERVER_HOST}`,
-      "authentication_ldap_sasl_server_port": parseInt(`${LDAP_SASL_SERVER_PORT}`),
-      "authentication_ldap_sasl_group_search_filter": `${LDAP_SASL_GROUP_SEARCH_FILTER}`,
-      "authentication_ldap_sasl_bind_base_dn": `${LDAP_SASL_BIND_BASE_DN}`,
-      "authentication_ldap_sasl_log_status": 5,
+      "loose-authentication_ldap_sasl_server_host": `${LDAP_SASL_SERVER_HOST}`,
+      "loose-authentication_ldap_sasl_server_port": parseInt(`${LDAP_SASL_SERVER_PORT}`),
+      "loose-authentication_ldap_sasl_group_search_filter": `${LDAP_SASL_GROUP_SEARCH_FILTER}`,
+      "loose-authentication_ldap_sasl_bind_base_dn": `${LDAP_SASL_BIND_BASE_DN}`,
+      "loose-authentication_ldap_sasl_log_status": 5,
       "log_error_verbosity": 3
     };
   } else if (context == 'LDAP_KERBEROS') {
     return {
       "plugin-load-add": `authentication_ldap_sasl.${ext}`,
-      "authentication_ldap_sasl_server_host": `${LDAP_KERBEROS_SERVER_HOST}`,
-      "authentication_ldap_sasl_server_port": parseInt(`${LDAP_KERBEROS_SERVER_PORT}`),
-      "authentication_ldap_sasl_bind_base_dn": `${LDAP_KERBEROS_BIND_BASE_DN}`,
-      "authentication_ldap_sasl_user_search_attr": `${LDAP_KERBEROS_USER_SEARCH_ATTR}`,
-      "authentication_ldap_sasl_bind_root_dn": `${LDAP_KERBEROS_BIND_ROOT_DN}`,
-      "authentication_ldap_sasl_bind_root_pwd": `${LDAP_KERBEROS_BIND_ROOT_PWD}`,
-      "authentication_ldap_sasl_group_search_filter": `${LDAP_KERBEROS_GROUP_SEARCH_FILTER}`,
-      "authentication_ldap_sasl_auth_method_name": 'GSSAPI',
-      "authentication_ldap_sasl_log_status": 5,
+      "loose-authentication_ldap_sasl_server_host": `${LDAP_KERBEROS_SERVER_HOST}`,
+      "loose-authentication_ldap_sasl_server_port": parseInt(`${LDAP_KERBEROS_SERVER_PORT}`),
+      "loose-authentication_ldap_sasl_bind_base_dn": `${LDAP_KERBEROS_BIND_BASE_DN}`,
+      "loose-authentication_ldap_sasl_user_search_attr": `${LDAP_KERBEROS_USER_SEARCH_ATTR}`,
+      "loose-authentication_ldap_sasl_bind_root_dn": `${LDAP_KERBEROS_BIND_ROOT_DN}`,
+      "loose-authentication_ldap_sasl_bind_root_pwd": `${LDAP_KERBEROS_BIND_ROOT_PWD}`,
+      "loose-authentication_ldap_sasl_group_search_filter": `${LDAP_KERBEROS_GROUP_SEARCH_FILTER}`,
+      "loose-authentication_ldap_sasl_auth_method_name": 'GSSAPI',
+      "loose-authentication_ldap_sasl_log_status": 5,
       "log_error_verbosity": 3,
       "net_read_timeout": 360,
       "connect_timeout": 360
@@ -334,8 +332,8 @@ function getAuthServerConfig(context) {
 
     return {
       "plugin-load-add": `authentication_kerberos.${ext}`,
-      "authentication_kerberos_service_principal": `mysql_service/kerberos_auth_host@${LDAP_KERBEROS_DOMAIN}`,
-      "authentication_kerberos_service_key_tab": keytab_file,
+      "loose-authentication_kerberos_service_principal": `mysql_service/kerberos_auth_host@${LDAP_KERBEROS_DOMAIN}`,
+      "loose-authentication_kerberos_service_key_tab": keytab_file,
       "net_read_timeout": 360,
       "connect_timeout": 360,
       "log_error_verbosity": 3
@@ -344,14 +342,19 @@ function getAuthServerConfig(context) {
     return {
       "plugin-load-add": `authentication_fido.${ext}`
     };
+  } else if (context == 'WEBAUTHN') {
+    return {
+      "loose-authentication-webauthn-rp-id": "mysql.com",
+      "plugin-load-add": `authentication_webauthn.${ext}`
+    };
   }
 
   return {};
 }
 
 
-function isAuthMethodSupported(context) {
-  if (['LDAP_SIMPLE', 'LDAP_SASL', 'LDAP_KERBEROS', 'KERBEROS', 'FIDO', 'OCI_AUTH'].indexOf(context) == -1) {
+function isAuthMethodSupported(context, uri=__mysqluripwd) {
+  if (['LDAP_SIMPLE', 'LDAP_SASL', 'LDAP_KERBEROS', 'KERBEROS', 'FIDO', 'WEBAUTHN', 'OCI_AUTH'].indexOf(context) == -1) {
     return false;
   }
 
@@ -367,18 +370,23 @@ function isAuthMethodSupported(context) {
     plugin = 'authentication_kerberos';
   } else if (context == 'FIDO') {
     plugin = 'authentication_fido';
+  } else if (context == 'WEBAUTHN') {
+    plugin = 'authentication_webauthn';
   } else if (context == 'OCI_AUTH') {
     // we're using an external server, so this is always supported
     return true;
   }
 
   var plugin_supported = true;
-  const s = shell.openSession(__mysqluripwd);
+  const s = shell.openSession(uri);
 
   try {
     ensure_plugin_enabled(plugin, s);
   } catch (error) {
-    plugin_supported = false;
+    plugin_supported = error.code == PLUGIN_ALREADY_EXISTS;
+    if (!plugin_supported) {
+      shell.log('warning', `Unable to get ${plugin} installed: Error ${error.code}: ${error.message}`)
+    }
   }
 
   if (plugin_supported) {
@@ -389,12 +397,6 @@ function isAuthMethodSupported(context) {
         plugin_supported = false;
       }
     }
-
-    try {
-      ensure_plugin_disabled(plugin, s);
-    } catch (error) {
-      // ignore any errors
-    }
   }
 
   s.close();
@@ -653,9 +655,9 @@ function disable_auto_rejoin(session, port) {
 
   if (__version_num > 80011) {
     try {
-      session.runSql("RESET PERSIST group_replication_start_on_boot");
+      session.runSql("SET PERSIST group_replication_start_on_boot=OFF");
     } catch (e) {
-      if (e["code"] != 3615) {
+      if (e.cause.code != 3615 && e.cause.code != 1193) {
         throw e;
       }
     }
@@ -723,8 +725,16 @@ function exist_in_metadata_schema(port) {
   return row[0] != 0;
 }
 
-function reset_instance(session) {
-  session.runSql("STOP SLAVE");
+function reset_instance(session, read_session_version) {
+  var version = __version_num;
+  if (read_session_version !== undefined && read_session_version) {
+    var version_array = session.runSql("SELECT version()").fetchOne()[0].match(/^(\d+)\.(\d+)\.(\d+)[^\d]*/);
+    EXPECT_EQ(4, version_array.length);
+
+    version = parseInt(version_array[1]) * 10000 + parseInt(version_array[2]) * 100 + parseInt(version_array[3]);
+  }
+
+  session.runSql("STOP " + get_replica_keyword(version));
   try {
   session.runSql("STOP group_replication");
   } catch (e) {}
@@ -749,18 +759,19 @@ function reset_instance(session) {
           continue;
       session.runSql("DROP USER ?@?", [row[0], row[1]]);
   }
-  session.runSql("RESET MASTER");
-  session.runSql("RESET SLAVE ALL");
+  session.runSql("RESET " + get_reset_binary_logs_keyword(version));
+  session.runSql("RESET " + get_replica_keyword(version) + " ALL");
 }
 
 var SANDBOX_PORTS = [__mysql_sandbox_port1, __mysql_sandbox_port2, __mysql_sandbox_port3];
 var SANDBOX_LOCAL_URIS = [__sandbox_uri1, __sandbox_uri2, __sandbox_uri3];
 var SANDBOX_URIS = [__hostname_uri1, __hostname_uri2, __hostname_uri3];
 
-var s = mysql.getSession(__mysqluripwd+"?ssl-mode=disabled");
+var s = mysql.getSession(__mysqluripwd+"?ssl-mode=preferred");
 var r = s.runSql("SELECT @@hostname, @@report_host").fetchOne();
 var __mysql_hostname = r[0];
 var __mysql_report_host = r[1];
+s.close()
 var __mysql_host = __mysql_report_host ? __mysql_report_host : __mysql_hostname;
 
 // Address that appear in pre-configured sandboxes that set report_host to 127.0.0.1
@@ -850,7 +861,7 @@ function begin_dba_log_sql(level) {
   var dummy = testutil.fetchDbaSqlLog(true);
 }
 
-function end_dba_log_sql(level) {
+function end_dba_log_sql() {
   var logs = testutil.fetchDbaSqlLog(false);
   shell.options["dba.logSql"] = 0;
   return logs;
@@ -934,6 +945,15 @@ function EXPECT_GT(value1, value2, note) {
   }
 }
 
+function EXPECT_GE(value1, value2, note) {
+  if (note === undefined)
+    note = "";
+  if (value1 < value2) {
+    var context = "<b>Context:</b> " + __test_context + "\n<red>EXPECT_GE failed:</red> " + note + "\n\t"+repr(value1)+" expected to be >= "+repr(value2)+" but isn't.";
+    testutil.fail(context);
+  }
+}
+
 function EXPECT_LT(value1, value2, note) {
   if (note === undefined)
     note = "";
@@ -943,6 +963,15 @@ function EXPECT_LT(value1, value2, note) {
   }
 }
 
+function EXPECT_LE(value1, value2, note) {
+  if (note === undefined)
+    note = "";
+  if (value1 > value2) {
+    var context = "<b>Context:</b> " + __test_context + "\n<red>EXPECT_LE failed:</red> " + note + "\n\t"+repr(value1)+" expected to be <= "+repr(value2)+" but isn't.";
+    testutil.fail(context);
+  }
+}
+
 function EXPECT_JSON_EQ(expected, actual, note) {
 
   function compare_values(expected, actual, path) {
@@ -1063,6 +1092,16 @@ function EXPECT_FALSE(value, note) {
   }
 }
 
+function get_exception_message(exception) {
+  if (exception.cause != undefined) {
+    if (exception.cause.message != undefined) {
+      return exception.cause.message;
+    }
+  }
+  return exception.message
+}
+
+
 function EXPECT_THROWS(func, etext) {
   if (typeof(etext) != "string" && typeof(etext) != "object") {
       testutil.fail("EXPECT_THROWS expects string or array, " +typeof(etext) + " given");
@@ -1071,15 +1110,17 @@ function EXPECT_THROWS(func, etext) {
     func();
     testutil.fail("<b>Context:</b> " + __test_context + "\n<red>Missing expected exception throw like " + etext + "</red>");
   } catch (err) {
-    testutil.dprint("Got exception as expected: " + JSON.stringify(err));
+    message = get_exception_message(err);
+    testutil.dprint("Got exception as expected: " + message);
+
     if (typeof(etext) === "string") {
-        if (err.message.indexOf(etext) < 0) {
-          testutil.fail("<b>Context:</b> " + __test_context + "\n<red>Exception expected:</red> " + etext + "\n\t<yellow>Actual:</yellow> " + err.message);
+        if (message.indexOf(etext) < 0) {
+          testutil.fail("<b>Context:</b> " + __test_context + "\n<red>Exception expected:</red> " + etext + "\n\t<yellow>Actual:</yellow> " + message);
         }
     } else if (typeof(etext) === "object") {
         var found = false;
         for (i in etext) {
-            if (err.message.indexOf(etext[i]) >= 0) {
+            if (message.indexOf(etext[i]) >= 0) {
                 found = true;
                 break;
             }
@@ -1089,7 +1130,7 @@ function EXPECT_THROWS(func, etext) {
             for (i in etext) {
                 msg += etext[i] + "\n";
             }
-            msg += "<yellow>Actual:</yellow> " + err.message;
+            msg += "<yellow>Actual:</yellow> " + message;
             testutil.fail(msg);
         }
     }
@@ -1105,12 +1146,26 @@ function EXPECT_THROWS_TYPE(func, etext, type) {
     if (err.message.indexOf(etext) < 0) {
       testutil.fail("<b>Context:</b> " + __test_context + "\n<red>Exception expected:</red> " + etext + "\n\t<yellow>Actual:</yellow> " + err.message);
     }
-    if (err.type  !== type) {
+
+    if (err.cause.type  !== type) {
       testutil.fail("<b>Context:</b> " + __test_context + "\n<red>Exception type expected:</red> " + type + "\n\t<yellow>Actual:</yellow> " + err.type);
     }
   }
 }
 
+function EXPECT_THROWS_LIKE(func, pattern) {
+  try {
+    func();
+    testutil.fail("<b>Context:</b> " + __test_context + "\n<red>Missing expected exception throw like " + String(etext) + "</red>");
+  } catch (err) {
+    msg = get_exception_message(err);
+    testutil.dprint("Caught exception: " + msg);
+    if (!pattern.test(msg)) {
+      testutil.fail("<b>Context:</b> " + __test_context + "\n<red>Exception expected:</red> " + String(pattern) + "\n\t<yellow>Actual:</yellow> " + err.message);
+    }
+  }
+}
+
 function EXPECT_NO_THROWS(func, context) {
   try {
     func();
@@ -1169,8 +1224,8 @@ function EXPECT_OUTPUT_EMPTY() {
 
 function EXPECT_STDOUT_CONTAINS(text) {
   var out = testutil.fetchCapturedStdout(false);
-  var err = testutil.fetchCapturedStderr(false);
   if (out.indexOf(text) < 0) {
+    var err = testutil.fetchCapturedStderr(false);
     var context = "<b>Context:</b> " + __test_context + "\n<red>Missing output:</red> " + text + "\n<yellow>Actual stdout:</yellow> " + out + "\n<yellow>Actual stderr:</yellow> " + err;
     testutil.fail(context);
   }
@@ -1178,7 +1233,6 @@ function EXPECT_STDOUT_CONTAINS(text) {
 
 function EXPECT_STDOUT_CONTAINS_ONE_OF(text) {
     var out = testutil.fetchCapturedStdout(false);
-    var err = testutil.fetchCapturedStderr(false);
     var found = false;
     for (i in text) {
         if (out.indexOf(text[i])>= 0) {
@@ -1187,6 +1241,7 @@ function EXPECT_STDOUT_CONTAINS_ONE_OF(text) {
         }
     }
     if (!found) {
+      var err = testutil.fetchCapturedStderr(false);
       var context = "<b>Context:</b> " + __test_context + "\n<red>Missing STDOUT output:</red>\n";
       for (i in text) {
           context += text[i] + "\n";
@@ -1197,7 +1252,6 @@ function EXPECT_STDOUT_CONTAINS_ONE_OF(text) {
 }
 
 function EXPECT_STDERR_CONTAINS_ONE_OF(text) {
-    var out = testutil.fetchCapturedStdout(false);
     var err = testutil.fetchCapturedStderr(false);
     var found = false;
     for (i in text) {
@@ -1207,6 +1261,7 @@ function EXPECT_STDERR_CONTAINS_ONE_OF(text) {
         }
     }
     if (!found) {
+      var out = testutil.fetchCapturedStdout(false);
       var context = "<b>Context:</b> " + __test_context + "\n<red>Missing STDERR output:</red>\n";
       for (i in text) {
           context += text[i] + "\n";
@@ -1217,9 +1272,9 @@ function EXPECT_STDERR_CONTAINS_ONE_OF(text) {
 }
 
 function EXPECT_STDERR_CONTAINS(text) {
-  var out = testutil.fetchCapturedStdout(false);
   var err = testutil.fetchCapturedStderr(false);
   if (err.indexOf(text) < 0) {
+    var out = testutil.fetchCapturedStdout(false);
     var context = "<b>Context:</b> " + __test_context + "\n<red>Missing error output:</red> " + text + "\n<yellow>Actual stdout:</yellow> " + out + "\n<yellow>Actual stderr:</yellow> " + err;
     testutil.fail(context);
   }
@@ -1452,23 +1507,23 @@ function EXPECT_OUTPUT_CONTAINS_MULTILINE(t) {
 
 function EXPECT_STDOUT_CONTAINS_MULTILINE(t) {
   const out = __split_trim_join(testutil.fetchCapturedStdout(false));
-  const err = __split_trim_join(testutil.fetchCapturedStderr(false));
   const text = __split_trim_join(t);
   const out_result = __check_multiline_expect(text.array, out.array);
 
   if (!out_result.matches) {
+    const err = __split_trim_join(testutil.fetchCapturedStderr(false));
     const context = "<b>Context:</b> " + __test_context + "\n<red>Missing output:</red> " + text.str + "\n<yellow>Actual stdout:</yellow> " + out.str + "\n<yellow>Actual stderr:</yellow> " + err.str + "\n<yellow>Diff with stdout:</yellow>\n" + out_result.diff;
     testutil.fail(context);
   }
 }
 
 function EXPECT_STDERR_CONTAINS_MULTILINE(t) {
-  const out = __split_trim_join(testutil.fetchCapturedStdout(false));
   const err = __split_trim_join(testutil.fetchCapturedStderr(false));
   const text = __split_trim_join(t);
   const err_result = __check_multiline_expect(text.array, err.array);
 
   if (!err_result.matches) {
+    const out = __split_trim_join(testutil.fetchCapturedStdout(false));
     const context = "<b>Context:</b> " + __test_context + "\n<red>Missing output:</red> " + text.str + "\n<yellow>Actual stdout:</yellow> " + out.str + "\n<yellow>Actual stderr:</yellow> " + err.str + "\n<yellow>Diff with stderr:</yellow>\n" + err_result.diff;
     testutil.fail(context);
   }
@@ -1485,17 +1540,17 @@ function EXPECT_OUTPUT_MATCHES(re) {
 
 function EXPECT_STDOUT_MATCHES(re) {
   var out = testutil.fetchCapturedStdout(false);
-  var err = testutil.fetchCapturedStderr(false);
   if (!re.test(out)) {
+    var err = testutil.fetchCapturedStderr(false);
     var context = "<b>Context:</b> " + __test_context + "\n<red>Missing match for:</red> " + re.source + "\n<yellow>Actual stdout:</yellow> " + out + "\n<yellow>Actual stderr:</yellow> " + err;
     testutil.fail(context);
   }
 }
 
 function EXPECT_STDERR_MATCHES(re) {
-  var out = testutil.fetchCapturedStdout(false);
   var err = testutil.fetchCapturedStderr(false);
   if (!re.test(err)) {
+    var out = testutil.fetchCapturedStdout(false);
     var context = "<b>Context:</b> " + __test_context + "\n<red>Missing match for:</red> " + re.source + "\n<yellow>Actual stdout:</yellow> " + out + "\n<yellow>Actual stderr:</yellow> " + err;
     testutil.fail(context);
   }
@@ -1573,17 +1628,17 @@ function EXPECT_OUTPUT_NOT_CONTAINS(text) {
 
 function EXPECT_STDOUT_NOT_CONTAINS(text) {
   var out = testutil.fetchCapturedStdout(false);
-  var err = testutil.fetchCapturedStderr(false);
   if (out.indexOf(text) >= 0) {
+    var err = testutil.fetchCapturedStderr(false);
     var context = "<b>Context:</b> " + __test_context + "\n<red>Unexpected output:</red> " + text + "\n<yellow>Actual stdout:</yellow> " + out + "\n<yellow>Actual stderr:</yellow> " + err;
     testutil.fail(context);
   }
 }
 
 function EXPECT_STDERR_NOT_CONTAINS(text) {
-  var out = testutil.fetchCapturedStdout(false);
   var err = testutil.fetchCapturedStderr(false);
   if (err.indexOf(text) >= 0) {
+    var out = testutil.fetchCapturedStdout(false);
     var context = "<b>Context:</b> " + __test_context + "\n<red>Unexpected output:</red> " + text + "\n<yellow>Actual stdout:</yellow> " + out + "\n<yellow>Actual stderr:</yellow> " + err;
     testutil.fail(context);
   }
@@ -1672,20 +1727,26 @@ function ClusterScenario(ports, create_cluster_options, sandboxConfiguration) {
 
   this.cluster = dba.createCluster("cluster", create_cluster_options);
 
-  var allowList = create_cluster_options["ipAllowlist"];
+  add_instance_options = {}
+  if ('ipAllowlist' in create_cluster_options) {
+    add_instance_options["ipAllowlist"] = create_cluster_options["ipAllowlist"];
+  }
+  if ('localAddress' in create_cluster_options) {
+    add_instance_options["localAddress"] = create_cluster_options["localAddress"];
+  }
 
   for (i in ports) {
     if (i > 0) {
       var uri = "mysql://root:root@localhost:" + ports[i];
-      if (allowList === undefined) {
+      if (Object.keys(add_instance_options).length === 0) {
         this.cluster.addInstance(uri);
       } else {
-        this.cluster.addInstance(uri, {ipAllowlist: allowList});
+        this.cluster.addInstance(uri, add_instance_options);
       }
       testutil.waitMemberState(ports[i], "ONLINE");
 
       if (testutil.versionCheck(__version, "<", "8.0.0")) {
-        dba.configureLocalInstance(uri, {mycnfPath: testutil.getSandboxConfPath(ports[i])});
+        dba.configureInstance(uri, {mycnfPath: testutil.getSandboxConfPath(ports[i])});
       }
     }
   }
@@ -1797,6 +1858,7 @@ function get_mysqlx_endpoint(uri) {
   return shell.unparseUri({ 'host': u.host, 'port': u.port * 10 });
 }
 
+var connection_error_msg = "Lost connection to MySQL server at 'waiting for initial communication packet', system error: 110";
 var protocol_error_msg = "The provided URI uses the X protocol, which is not supported by this command.";
 
 function EXPECT_THROWS_ERROR(msg, f, ...args) {
@@ -1810,14 +1872,15 @@ function CHECK_MYSQLX_EXPECT_THROWS_ERROR(msg, f, classic, ...args) {
 }
 
 function EXPECT_DBA_THROWS_PROTOCOL_ERROR(context, f, classic, ...args) {
-  CHECK_MYSQLX_EXPECT_THROWS_ERROR(`${context}: ${protocol_error_msg}`, f, classic, ...args);
+  CHECK_MYSQLX_EXPECT_THROWS_ERROR([`${context}: ${protocol_error_msg}`, `${context}: ${connection_error_msg}`], f, classic, ...args);
 }
 
 function EXPECT_CLUSTER_THROWS_PROTOCOL_ERROR(context, f, classic, ...args) {
   for (const uri of get_mysqlx_uris(classic)) {
     const endpoint = get_mysqlx_endpoint(classic);
     WIPE_OUTPUT();
-    EXPECT_THROWS_ERROR(`${context}: Could not open connection to '${endpoint}': ${protocol_error_msg}`, f, uri, ...args);
+    EXPECT_THROWS_ERROR([`${context}: Could not open connection to '${endpoint}': ${protocol_error_msg}`,
+        `${context}: Could not open connection to '${endpoint}': ${connection_error_msg}`], f, uri, ...args);
     EXPECT_STDOUT_CONTAINS(`Unable to connect to the target instance '${endpoint}'. Please verify the connection settings, make sure the instance is available and try again.`);
   }
 }
@@ -1875,3 +1938,178 @@ function CHECK_TABLE_SNAPSHOTS(session, snapshot) {
     }
   }
 }
+// Standard Checks
+
+function STDCHECK_ARGTYPES(func, min_args, good_args1, good_args2, good_args3, good_args4, good_args5) {
+  // Usage
+  // Given: myfunction(string, int[, int]), where arg1 can also be null
+  // 
+  // STDCHECK_ARGTYPES(myfunction, 2, ["abc", null], [42], [42])
+  // 
+
+  bad_values = {
+      "number": ["hello", [1], {"a":1}, testutil, false],
+      "boolean": ["hello", [1], {"a":1}, testutil], // number ok
+      "string": [1234, ["a"], {"a":"b"}, testutil],
+      "array": ["hello", 1234, {"a":"b"}, testutil],
+      "map": ["hello", 1234, ["a"], testutil]
+  };
+
+  function type_name(arg) {
+      t = typeof arg;
+      if (t == "object" && Array.isArray(arg))
+          t = "an array";
+      else if (t == "object")
+          t = "a map";
+      else
+          t = "a " + t
+      return t;
+  }
+
+  function get_bad_values(args) {
+    var t = typeof args[0];
+    if (t == "object" && Array.isArray(args[0]))
+        t = "array";
+    else if (t == "object")
+        t = "map";
+    var values = bad_values[t];
+    if (args.indexOf(null) >= 0)
+      values.push(null);
+    return values;
+  }
+
+  // check too few args
+  if (min_args >= 1)
+      EXPECT_THROWS(function(){ func(); }, "Invalid number of arguments");
+  if (min_args >= 2)
+      EXPECT_THROWS(function(){ func(good_args1[0]); }, "Invalid number of arguments");
+  if (min_args >= 3)
+      EXPECT_THROWS(function(){ func(good_args1[0], good_args2[0]); }, "Invalid number of arguments");
+  if (min_args >= 4)
+      EXPECT_THROWS(function(){ func(good_args1[0], good_args2[0], good_args3[0]); }, "Invalid number of arguments");
+  if (min_args >= 5)
+      EXPECT_THROWS(function(){ func(good_args1[0], good_args2[0], good_args3[0], good_args4[0]); }, "Invalid number of arguments");
+  
+  // check bad arg types (this test doesn't support alternative typing)
+  if (good_args5 === undefined) {
+      if (good_args4 === undefined) {
+          if (good_args3 === undefined) {
+              if (good_args2 === undefined) {
+                  // 1 arg
+                  for (arg1 of get_bad_values(good_args1))
+                      EXPECT_THROWS(function() { func(arg1); }, "Argument #1 is expected to be "+type_name(good_args1[0]));
+
+                  EXPECT_THROWS(function(){ func(good_args1[0], "extra"); }, "Invalid number of arguments");
+              } else {
+                  // 2 args
+                  for (arg2 of get_bad_values(good_args2))
+                      EXPECT_THROWS(function() { func(good_args1[0], arg2); }, "Argument #2 is expected to be "+type_name(good_args2[0]));
+
+                  for (arg1 of get_bad_values(good_args1)) 
+                      EXPECT_THROWS(function() { func(arg1, good_args2[0]); }, "Argument #1 is expected to be "+type_name(good_args1[0]));
+
+                  EXPECT_THROWS(function(){ func(good_args1[0], good_args2[0], "extra"); }, "Invalid number of arguments");
+              }
+          } else {
+              // 3 args
+              for (arg3 of get_bad_values(good_args3))
+                  EXPECT_THROWS(function() { func(good_args1[0], good_args2[0], arg3); }, "Argument #3 is expected to be "+type_name(good_args3[0]));
+
+              for (arg2 of get_bad_values(good_args2))
+                  EXPECT_THROWS(function() { func(good_args1[0], arg2, good_args3[0]); }, "Argument #2 is expected to be "+type_name(good_args2[0]));
+
+              for (arg1 of get_bad_values(good_args1)) 
+                  EXPECT_THROWS(function() { func(arg1, good_args2[0], good_args3[0]); }, "Argument #1 is expected to be "+type_name(good_args1[0]));
+
+              EXPECT_THROWS(function(){ func(good_args1[0], good_args2[0], good_args3[0], "extra"); }, "Invalid number of arguments");
+          }
+      } else {
+          // 4 args
+          for (arg4 of get_bad_values(good_args4))
+              EXPECT_THROWS(function() { func(good_args1[0], good_args2[0], good_args3[0], arg4); }, "Argument #4 is expected to be "+type_name(good_args4[0]));
+
+          for (arg3 of get_bad_values(good_args3))
+              EXPECT_THROWS(function() { func(good_args1[0], good_args2[0], arg3, good_args4[0]); }, "Argument #3 is expected to be "+type_name(good_args3[0]));
+
+          for (arg2 of get_bad_values(good_args2))
+              EXPECT_THROWS(function() { func(good_args1[0], arg2, good_args3[0], good_args4[0]); }, "Argument #2 is expected to be "+type_name(good_args2[0]));
+
+          for (arg1 of get_bad_values(good_args1)) 
+              EXPECT_THROWS(function() { func(arg1, good_args2[0], good_args3[0], good_args4[0]); }, "Argument #1 is expected to be "+type_name(good_args1[0]));
+
+          EXPECT_THROWS(function(){ func(good_args1[0], good_args2[0], good_args3[0], good_args4[0], "extra"); }, "Invalid number of arguments");
+      }
+  } else {
+      // 5 args
+      for (arg5 of get_bad_values(good_args5))
+          EXPECT_THROWS(function() { func(good_args1[0], good_args2[0], good_args3[0], good_args4[0], arg5); }, "Argument #5 is expected to be "+type_name(good_args5[0]));
+
+      for (arg4 of get_bad_values(good_args4))
+          EXPECT_THROWS(function() { func(good_args1[0], good_args2[0], good_args3[0], arg4); }, "Argument #4 is expected to be "+type_name(good_args4[0]));
+
+      for (arg3 of get_bad_values(good_args3))
+          EXPECT_THROWS(function() { func(good_args1[0], good_args2[0], arg3, good_args4[0]); }, "Argument #3 is expected to be "+type_name(good_args3[0]));
+
+      for (arg2 of get_bad_values(good_args2))
+          EXPECT_THROWS(function() { func(good_args1[0], arg2, good_args3[0], good_args4[0]); }, "Argument #2 is expected to be "+type_name(good_args2[0]));
+
+      for (arg1 of get_bad_values(good_args1)) 
+          EXPECT_THROWS(function() { func(arg1, good_args2[0], good_args3[0], good_args4[0]); }, "Argument #1 is expected to be "+type_name(good_args1[0]));
+
+      EXPECT_THROWS(function(){ func(good_args1[0], good_args2[0], good_args3[0], good_args4[0], good_args5[0], "extra"); }, "Invalid number of arguments");
+  }
+}
+
+function get_reset_binary_logs_keyword(version) {
+  if (version === undefined) {
+    version = __version_num;
+  }
+
+  if (version < 80200) {
+    return "MASTER";
+  }
+
+  return "BINARY LOGS AND GTIDS";
+}
+
+function get_replica_keyword(version) {
+  if (version === undefined)
+    version = __version_num;
+
+  if (version < 80022) {
+    return "SLAVE";
+  }
+
+  return "REPLICA";
+}
+
+var replication_source_keywords = {
+  "REPLICATION SOURCE": "MASTER",
+  "GET_SOURCE_PUBLIC_KEY": "GET_MASTER_PUBLIC_KEY",
+  "SOURCE_USER": "MASTER_USER",
+  "SOURCE_PASSWORD": "MASTER_PASSWORD",
+  "REPLICA": "SLAVE"
+};
+
+function get_replication_source_keyword(version=__version_num, keyword="REPLICATION SOURCE") {
+  if (!(keyword in replication_source_keywords)) {
+    throw `Unkwnown replication source keyword: ${keyword}`
+  }
+
+  if (version < 80022) {
+    return replication_source_keywords[keyword];
+  }
+
+  return keyword;
+}
+
+function get_replication_option_keyword(version) {
+  if (version === undefined)
+    version = __version_num;
+
+  if (version < 80022) {
+    return "MASTER";
+  }
+
+  return "SOURCE";
+}
