@@ -11,6 +11,7 @@ import re
 import string
 import sys
 import inspect
+import json
 
 
 mysqlshrec = "mysqlshrec"
@@ -42,6 +43,119 @@ def get_members(object):
 ##
 
 
+def __split_trim_join(text):
+    needle = '\n'
+    s = [t.rstrip() for t in text.split(needle)]
+    return {'str': needle.join(s), 'array': s}
+
+
+def __check_wildcard_match(expected, actual):
+    if 0 == len(expected):
+        return expected == actual
+
+    needle = '[[*]]'
+    strings = expected.split(needle)
+    start = 0
+
+    for line in strings:
+        idx = actual.find(line, start)
+
+        if idx < 0:
+            return False
+
+        start = idx + len(line)
+
+    if strings[-1] == '':
+        # expected ends with wildcard
+        start = len(actual)
+
+    return start == len(actual)
+
+
+def __multi_value_compare(expected, actual):
+    start = expected.find('{{')
+
+    if (start < 0):
+        return __check_wildcard_match(expected, actual)
+    else:
+        end = expected.find('}}')
+        pre = expected[0, start]
+        post = expected[end + 2]
+        opts = expected[start + 2, end]
+
+        for item in opts.split('|'):
+            if __check_wildcard_match(pre + item + post, actual):
+                return True
+
+        return False
+
+
+def __find_line_matching(expected, actual_lines, start_at=0):
+    actual_index = start_at
+
+    while actual_index < len(actual_lines):
+        if __multi_value_compare(expected, actual_lines[actual_index]):
+            break
+        else:
+            actual_index = actual_index + 1
+
+    return actual_index
+
+
+def __diff_with_error(arr, error, idx):
+    needle = '\n'
+    copy = arr.copy()
+    copy[idx] += error
+    return needle.join(copy)
+
+
+def __check_multiline_expect(expected_lines, actual_lines):
+    diff = ''
+    matches = True
+    needle = '\n'
+
+    while '' == expected_lines[0] or '[[*]]' == expected_lines[0]:
+        expected_lines.pop(0)
+
+    actual_index = __find_line_matching(expected_lines[0], actual_lines)
+
+    if actual_index < len(actual_lines):
+        for expected_index in range(len(expected_lines)-1):
+            if '[[*]]' == expected_lines[expected_index]:
+                if expected_index == len(expected_lines) - 1:
+                    continue  # Ignores [[*]] if at the end of the expectation
+                else:
+                    expected_index = expected_index + 1
+
+                    actual_index = __find_line_matching(
+                        expected_lines[expected_index], actual_lines, actual_index + expected_index - 1)
+
+                    if actual_index < len(actual_lines):
+                        actual_index = actual_index - expected_index
+                    else:
+                        matches = False
+                        diff = __diff_with_error(
+                            expected_lines, '<yellow><------ INCONSISTENCY</yellow>', expected_index)
+                        break
+
+            if (actual_index + expected_index) >= len(actual_lines):
+                matches = False
+                diff = __diff_with_error(
+                    expected_lines, '<yellow><------ MISSING</yellow>', expected_index)
+
+            if not __multi_value_compare(expected_lines[expected_index], actual_lines[actual_index + expected_index]):
+                matches = False
+                diff = __diff_with_error(
+                    expected_lines, '<yellow><------ INCONSISTENCY</yellow>', expected_index)
+                break
+    else:
+        matches = False
+        diff = __diff_with_error(
+            expected_lines, '<yellow><------ INCONSISTENCY</yellow>', 0)
+
+    return {'matches': matches, 'diff': diff}
+
+
 def defined(cb):
     try:
         cb()
@@ -276,6 +390,39 @@ def EXPECT_DELTA(expected, allowed_delta, actual, note=""):
                    allowed_delta, actual, note)
 
 
+def EXPECT_JSON_EQ(expected, actual, note = ""):
+    def json_to_string(o):
+        return json.dumps(o, indent = 2)
+    def compare_values(expected, actual, path):
+        if type(expected) != type(actual):
+            print(" =>", path, "mismatched values. Expected:\n", expected, "\nActual:\n", actual, "\n")
+            return 1
+        elif isinstance(expected, dict):
+            return compare_objects(expected, actual, path)
+        else:
+            jexpected = json_to_string(expected)
+            jactual = json_to_string(actual)
+            if jexpected != jactual:
+                print(" =>", path, "mismatched values. Expected:\n", jexpected, "\nActual:\n", jactual, "\n")
+                return 1
+        return 0
+    def compare_objects(expected, actual, path):
+        ndiffs = 0
+        for k in expected.keys():
+            if k not in actual:
+                ndiffs += 1
+                print(" =>", path + "." + k, "expected but missing from result")
+            else:
+                ndiffs += compare_values(expected[k], actual[k], path + "." + k)
+        for k in actual.keys():
+            if k not in expected:
+                ndiffs += 1
+                print(" =>", path + "." + k, "unexpected but found in result")
+        return ndiffs
+    if compare_values(expected, actual, "$") > 0:
+        testutil.fail(f"<b>Context:</b> {__test_context}\n<red>Tested values don't match as expected.</red> {note}\n\t<yellow>Actual:</yellow> {json_to_string(actual)}\n\t<yellow>Expected:</yellow> {json_to_string(expected)}")
+
+
 def EXPECT_TRUE(value, note=""):
     if not value:
         if not note:
@@ -351,6 +498,23 @@ def EXPECT_STDOUT_CONTAINS(text, note=None):
             "\n<yellow>Actual stderr:</yellow> " + err
         testutil.fail(context)
 
+def EXPECT_STDOUT_CONTAINS_ONE_OF(expected_list, note=None):
+    out = testutil.fetch_captured_stdout(False)
+    err = testutil.fetch_captured_stderr(False)
+
+    found = False
+    for item in expected_list:
+        if out.find(item) != -1:
+            found = True
+
+    if not found:
+        if not note:
+            note = __caller_context()
+        context = "<b>Context:</b> " + __test_context + "\n<red>Missing output (Any of the following):</red> " + "\n".join(expected_list) + \
+            "\n<yellow>Actual stdout:</yellow> " + out + \
+            "\n<yellow>Actual stderr:</yellow> " + err
+        testutil.fail(context)
+
 
 def EXPECT_STDERR_CONTAINS(text, note=None):
     out = testutil.fetch_captured_stdout(False)
@@ -449,6 +613,17 @@ def EXPECT_STDOUT_NOT_CONTAINS(text, note=None):
             "\n<yellow>Actual stderr:</yellow> " + err
         testutil.fail(context)
 
+def EXPECT_STDERR_NOT_CONTAINS(text, note=None):
+    out = testutil.fetch_captured_stdout(False)
+    err = testutil.fetch_captured_stderr(False)
+    if err.find(text) != -1:
+        if not note:
+            note = __caller_context()
+        context = "<b>Context:</b> " + __test_context + "\n<red>Missing output:</red> " + text + \
+            "\n<yellow>Actual stdout:</yellow> " + out + \
+            "\n<yellow>Actual stderr:</yellow> " + err
+        testutil.fail(context)
+
 
 def EXPECT_FILE_CONTAINS(expected, path, note=None):
     with open(path, encoding='utf-8') as f:
@@ -652,7 +827,7 @@ def reset_instance(session):
       session.run_sql("SELECT asynchronous_connection_failover_reset()")
     except:
         pass
-    session.run_sql("STOP SLAVE")
+    session.run_sql("STOP " + get_replica_keyword())
     try:
         session.run_sql("STOP group_replication")
         session.run_sql("SET PERSIST group_replication_start_on_boot=0")
@@ -675,8 +850,8 @@ def reset_instance(session):
         if row[0] == "root" and (row[1] == "localhost" or row[1] == "%"):
             continue
         session.run_sql("DROP USER ?@?", [row[0], row[1]])
-    session.run_sql("RESET MASTER")
-    session.run_sql("RESET SLAVE ALL")
+    session.run_sql("RESET " + get_reset_binary_logs_keyword())
+    session.run_sql("RESET " + get_replica_keyword() + " ALL")
 
 
 def reset_multi(ports):
@@ -788,3 +963,69 @@ class Docker_manipulator:
             self.client.containers.prune()
             if remove_image:
                 self.client.images.remove(image=self.img.id, force=True)
+
+def get_reset_binary_logs_keyword():
+  if __version_num < 80200:
+    return "MASTER"
+
+  return "BINARY LOGS AND GTIDS"
+
+def get_replica_keyword():
+  if __version_num < 80022:
+    return "SLAVE"
+
+  return "REPLICA"
+
+def get_replication_source_keyword():
+  if __version_num < 80022:
+    return "MASTER"
+
+  return "REPLICATION SOURCE"
+
+def get_replication_option_keyword():
+  if __version_num < 80022:
+    return "MASTER"
+
+  return "SOURCE"
+
+def EXPECT_OUTPUT_CONTAINS_MULTILINE(t):
+    out = __split_trim_join(testutil.fetch_captured_stdout(False))
+    err = __split_trim_join(testutil.fetch_captured_stderr(False))
+    text = __split_trim_join(t)
+    out_result = __check_multiline_expect(text["array"], out["array"])
+    err_result = __check_multiline_expect(text["array"], err["array"])
+
+    if not out_result["matches"] and not err_result["matches"]:
+        context = "<b>Context:</b> " + __test_context + "\n<red>Missing output:</red> " + text["str"] + "\n<yellow>Actual stdout:</yellow> " + out["str"] + \
+            "\n<yellow>Actual stderr:</yellow> " + err["str"] + "\n<yellow>Diff with stdout:</yellow>\n" + \
+            out_result["diff"] + \
+            "\n<yellow>Diff with stderr:</yellow>\n" + err_result["diff"]
+        testutil.fail(context)
+
+
+def EXPECT_STDOUT_CONTAINS_MULTILINE(t):
+    out = __split_trim_join(testutil.fetch_captured_stdout(False))
+    err = __split_trim_join(testutil.fetch_captured_stderr(False))
+    text = __split_trim_join(t)
+    out_result = __check_multiline_expect(text["array"], out["array"])
+
+    if not out_result["matches"]:
+        context = "<b>Context:</b> " + __test_context + "\n<red>Missing output:</red> " + \
+            text["str"] + "\n<yellow>Actual stdout:</yellow> " + out["str"] + "\n<yellow>Actual stderr:</yellow> " + \
+            err["str"] + "\n<yellow>Diff with stdout:</yellow>\n" + \
+            out_result["diff"]
+        testutil.fail(context)
+
+
+def EXPECT_STDERR_CONTAINS_MULTILINE(t):
+    out = __split_trim_join(testutil.fetchCapturedStdout(False))
+    err = __split_trim_join(testutil.fetchCapturedStderr(False))
+    text = __split_trim_join(t)
+    err_result = __check_multiline_expect(text["array"], err["array"])
+
+    if not err_result["matches"]:
+        context = "<b>Context:</b> " + __test_context + "\n<red>Missing output:</red> " + \
+            text["str"] + "\n<yellow>Actual stdout:</yellow> " + out["str"] + "\n<yellow>Actual stderr:</yellow> " + \
+            err["str"] + "\n<yellow>Diff with stderr:</yellow>\n" + \
+            err_result["diff"]
+        testutil.fail(context)
