@@ -175,21 +175,32 @@ void detect_mysql_environment(int port, const char *pwd) {
     }
 
     {
-      const char *query =
-          "select @@version, @@have_ssl = 'YES', @@have_openssl = 'YES', "
-          "@@server_id";
+      const char *query = "select @@version, @@server_id";
       if (mysql_real_query(mysql, query, strlen(query)) == 0) {
         MYSQL_RES *res = mysql_store_result(mysql);
         if (MYSQL_ROW row = mysql_fetch_row(res)) {
           g_target_server_version = mysqlshdk::utils::Version(row[0]);
-          if (row[1] && strcmp(row[1], "1") == 0) have_ssl = true;
-          if (row[2] && strcmp(row[2], "1") == 0) have_openssl = true;
-          server_id = atoi(row[3]);
+          server_id = atoi(row[1]);
         }
         mysql_free_result(res);
       }
     }
 
+    if (g_target_server_version < mysqlshdk::utils::Version(8, 0, 24)) {
+      const char *query = "select @@have_ssl = 'YES', @@have_openssl = 'YES'";
+      if (mysql_real_query(mysql, query, strlen(query)) == 0) {
+        MYSQL_RES *res = mysql_store_result(mysql);
+        if (MYSQL_ROW row = mysql_fetch_row(res)) {
+          if (row[0] && strcmp(row[0], "1") == 0) have_ssl = true;
+          if (row[1] && strcmp(row[1], "1") == 0) have_openssl = true;
+        }
+        mysql_free_result(res);
+      }
+    } else {
+      have_ssl = true;
+      have_openssl = true;
+    }
+
     {
       const char *query = "select @@mysqlx_port";
       if (mysql_real_query(mysql, query, strlen(query)) == 0) {
@@ -303,16 +314,26 @@ void detect_mysql_environment(int port, const char *pwd) {
     // compiled-in socket path with the actual path in use
     shcore::setenv("MYSQL_UNIX_PORT", socket_absolute);
   }
+  {
+    // This environment variable makes libmysqlclient override the default
+    // compiled-in TCP port with the actual one in use
+    if (!shcore::setenv("MYSQL_TCP_PORT", std::to_string(port))) {
+      std::cerr << "MYSQL_TCP_PORT putenv failed to set it\n";
+      exit(1);
+    }
+  }
 
   {
+    // does not affect libmysqlx
     if (!shcore::setenv("MYSQLX_SOCKET", xsocket_absolute)) {
       std::cerr << "MYSQLX_SOCKET putenv failed to set it\n";
       exit(1);
     }
   }
   {
+    // does not affect libmysqlx
     if (!shcore::setenv("MYSQLX_PORT", std::to_string(xport))) {
-      std::cerr << "MYSQLX_SOCKET putenv failed to set it\n";
+      std::cerr << "MYSQLX_PORT putenv failed to set it\n";
       exit(1);
     }
   }
@@ -490,7 +511,8 @@ bool delete_sandbox(int port) {
 
   // Wait for the running server to complete the shutdown sequence
 
-  std::cout << "Waiting for server running at " << port << " to shut down\n";
+  std::cout << "Waiting for server running at " << port << " to shut down..."
+            << std::endl;
 
   int retries = k_wait_sandbox_shutdown;
 
@@ -507,7 +529,8 @@ bool delete_sandbox(int port) {
   // Wait for the ports (classic, x and xcom) to be freed
   while (is_port_listening("127.0.0.1", port)) {
     if (--retries < 0) {
-      std::cout << "Timeout waiting for port " << port << " to be free";
+      std::cout << "Timeout waiting for port " << port << " to be free."
+                << std::endl;
       return false;
     }
     shcore::sleep_ms(1000);
@@ -517,7 +540,8 @@ bool delete_sandbox(int port) {
   retries = k_wait_sandbox_shutdown;
   while (is_port_listening("127.0.0.1", x_port)) {
     if (--retries < 0) {
-      std::cout << "Timeout waiting for port " << x_port << " to be free";
+      std::cout << "Timeout waiting for port " << x_port << " to be free."
+                << std::endl;
       return false;
     }
     shcore::sleep_ms(1000);
@@ -527,7 +551,8 @@ bool delete_sandbox(int port) {
   retries = k_wait_sandbox_shutdown;
   while (is_port_listening("127.0.0.1", xcom_port)) {
     if (--retries < 0) {
-      std::cout << "Timeout waiting for port " << xcom_port << " to be free";
+      std::cout << "Timeout waiting for port " << xcom_port << " to be free."
+                << std::endl;
       return false;
     }
     shcore::sleep_ms(1000);
@@ -543,7 +568,7 @@ bool delete_sandbox(int port) {
     while (mysqlshdk::utils::check_lock_file(lock_file)) {
       if (--retries < 0) {
         std::cout << "Timeout waiting for sandbox lock file " << lock_file
-                  << " to be deleted after shutdown ";
+                  << " to be deleted after shutdown." << std::endl;
         return false;
       }
       shcore::sleep_ms(1000);
@@ -555,7 +580,7 @@ bool delete_sandbox(int port) {
     while (mysqlshdk::utils::check_lock_file(xlock_file, "X%zd")) {
       if (--retries < 0) {
         std::cout << "Timeout waiting for sandbox lock file " << xlock_file
-                  << " to be deleted after shutdown ";
+                  << " to be deleted after shutdown." << std::endl;
         return false;
       }
       shcore::sleep_ms(1000);
@@ -569,7 +594,7 @@ bool delete_sandbox(int port) {
     while (shcore::path::exists(pidfile)) {
       if (--retries < 0) {
         std::cout << "Timeout waiting for sandbox pid file " << pidfile
-                  << " to be deleted after shutdown ";
+                  << " to be deleted after shutdown." << std::endl;
         return false;
       }
       shcore::sleep_ms(1000);
@@ -580,10 +605,10 @@ bool delete_sandbox(int port) {
     if (shcore::is_folder(d)) {
       try {
         shcore::remove_directory(d, true);
-        std::cout << "Deleted leftover sandbox dir " << d << "\n";
+        std::cout << "Deleted leftover sandbox dir " << d << std::endl;
       } catch (std::exception &e) {
         std::cout << "Error deleting sandbox dir " << d << ": " << e.what()
-                  << "\n";
+                  << std::endl;
         return false;
       }
     }
@@ -599,7 +624,7 @@ bool delete_sandbox(int port) {
     std::cout << "Terminating sandbox process\n";
 
     if (system(s.c_str()) != 0) {
-      std::cout << "Error: unable to terminate sandbox process\n";
+      std::cout << "Error: unable to terminate sandbox process." << std::endl;
     }
   }
 #endif
@@ -1144,7 +1169,7 @@ int main(int argc, char **argv) {
   }
   {
     std::cout << "mycnf defaults: "
-              << system((mysqlsh_path + " --print-defaults").c_str());
+              << system((mysqlsh_path + " --print-defaults").c_str()) << "\n";
   }
 
   switch (g_test_recording_mode) {
